package com.yunzhi.nanyang.common;

import cn.dev33.satoken.stp.StpUtil;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.service.ISysUserDataScopeService;
import com.yunzhi.nanyang.service.ISysUserService;
import com.yunzhi.nanyang.service.ITaPersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public class BaseController {

    @Autowired
    protected ISysUserService iSysUserService;

    @Autowired
    protected ITaPersonService iTaPersonService;

    @Autowired
    protected ISysUserDataScopeService iSysUserDataScopeService;

    public String getLoginId() {
        return StpUtil.getLoginIdAsString();
    }

    public SysUser currentUser() throws Exception {
        String loginId = getLoginId();
        if (StringUtils.isEmpty(loginId)) {
            throw new Exception("请先登录");
        }

        SysUser user = iSysUserService.getByLoginId(loginId);
        if (user == null || user.getStatus() == Constants.STATUS_DELETE) {
            throw new Exception("人员不存在");
        }

        return user;
    }

    public TaPerson currentPerson() throws Exception {
        String personId = getLoginId();
        if (StringUtils.isEmpty(personId)) {
            throw new Exception("请先登录");
        }

        TaPerson taPerson = iTaPersonService.getById(personId);
        if (taPerson == null || taPerson.getStatus() == Constants.STATUS_DELETE) {
            throw new Exception("人员不存在");
        }

        return taPerson;
    }

    public void checkOrgAccess(String orgId) throws Exception {
        SysUser user = currentUser();

        // 如果是管理员
        if (Constants.ADMIN_ID.equals(user.getUserId())) return;

        // 空机构
        if (StringUtils.isEmpty(orgId)) return;

        boolean canAccess = iSysUserDataScopeService.checkOrgAccess(orgId, user.getUserId());
        if (!canAccess) {
            throw new Exception("暂无机构数据权限");
        }
    }

    public SysUser checkWorker(TaPerson taPerson) throws Exception {
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            throw new Exception("未绑定农机手");
        }

        SysUser sysUser = iSysUserService.getExistBy("user_id", taPerson.getUserId(), false, true);
        if (null == sysUser) {
            throw new Exception("农机手信息不存在");
        }

        return sysUser;
    }

    public void checkMClient(String client) throws Exception {
        if (StringUtils.isEmpty(client)) {
            throw new Exception("暂不支持的客户端");
        }

        if (!"android".equals(client) && !"ios".equals(client)) {
            throw new Exception("暂不支持的客户端");
        }
    }

    public List<String> getAccessOrgs(String userId) {
        return iSysUserDataScopeService.getAccessOrgs(userId);
    }
}
