package com.yunzhi.nanyang.common;

import com.github.binarywang.wxpay.bean.request.WxPayRefundV3Request;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderV3Request;
import com.github.binarywang.wxpay.bean.result.WxPayRefundV3Result;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderV3Result;
import com.github.binarywang.wxpay.bean.result.enums.TradeTypeEnum;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import com.yunzhi.nanyang.config.WeixinConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.ZoneOffset;

@Component
public class WxPayUtils {
    @Autowired
    WeixinConfig config;

    @Autowired
    WxUtils wxUtils;

    @Value("${yz.default.servBase}")
    String servBase;

    /**
     * 下单
     * @param request
     * @return
     * @throws Exception
     */
    public WxPayUnifiedOrderV3Result createOrder(WxPayUnifiedOrderV3Request request, String client) throws WxPayException {
        // 北京时间
        LocalDateTime now = LocalDateTime.now(ZoneOffset.ofHours(8));
        request.setNotifyUrl(servBase + "/wxpay/notify/"+client+"/order");

        if ("ios".equals(client) || "android".equals(client)) {
            WxPayService payService = wxUtils.getAndroidPayService();
            return payService.unifiedOrderV3(TradeTypeEnum.APP, request);
        } else {
            WxPayService payService = wxUtils.getMiniPayService();
            return payService.unifiedOrderV3(TradeTypeEnum.JSAPI, request);
        }

//        WxPayUnifiedOrderV3Result wxPayUnifiedOrderV3Result =
//        return wxPayUnifiedOrderV3Result.getPayInfo(tradeType, request.getAppid(), request.getMchid(), payService.getConfig().getPrivateKey());
    }

    /**
     * 订单退款
     * @param request
     * @return
     * @throws Exception
     */
    public WxPayRefundV3Result refund(WxPayRefundV3Request request, String client) throws Exception {
        WxPayService payService = null;
        if ("ios".equals(client) || "android".equals(client)) {
            payService = wxUtils.getAndroidPayService();
        } else {
            payService = wxUtils.getMiniPayService();
        }

        request.setNotifyUrl(servBase + "/wxpay/notify/"+client+"/refund");
        WxPayRefundV3Result result = payService.refundV3(request);

        if ("ABNORMAL".equals(result.getStatus())) {
            throw new Exception("退款异常");
        }
        if ("CLOSED".equals(result.getStatus())) {
            throw new Exception("退款关闭");
        }

        return result;
    }

}
