package com.yunzhi.nanyang.common.alipay;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.CertAlipayRequest;
import com.alipay.api.DefaultAlipayClient;
import lombok.Data;

/**
 * 主要用来构造 支付宝支付客户端
 */
public class AlipayFactory {

    Config config;
    AlipayClient alipayClient;

    public AlipayFactory(Config config) {
        this.config = config;

        try {
            this.initAlipayClient();
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
    }

    public AlipayClient getAlipayClient() throws AlipayApiException {
        if (alipayClient == null) {
            this.initAlipayClient();
        }

        return alipayClient;
    }

    private void initAlipayClient() throws AlipayApiException {
        CertAlipayRequest certAlipayRequest = new CertAlipayRequest();
        //设置网关地址
        certAlipayRequest.setServerUrl(config.getServerUrl());
        //设置应用Id
        certAlipayRequest.setAppId(config.getAppid());
        //设置应用私钥
        certAlipayRequest.setPrivateKey(config.getPrivateKey());
        //设置请求格式，固定值json
        certAlipayRequest.setFormat(config.getFormat());
        //设置字符集
        certAlipayRequest.setCharset(config.getCharset());
        //设置签名类型
        certAlipayRequest.setSignType(config.getSignType());
        //设置应用公钥证书路径
        certAlipayRequest.setCertPath(config.getCertPath());
        //设置支付宝公钥证书路径
        certAlipayRequest.setAlipayPublicCertPath(config.getAlipayPublicCertPath());
        //设置支付宝根证书路径
        certAlipayRequest.setRootCertPath(config.getRootCertPath());
        //构造client
        alipayClient = new DefaultAlipayClient(certAlipayRequest);
    }

    @Data
    public static class Config {
        String appid;
        String privateKey;
        String certPath;
        String alipayPublicCertPath;
        String rootCertPath;
        String format;
        String charset;
        String signType;
        String serverUrl;
        String notifyUrl;
        String authUrl;
        String authCallback;
    }

}
