package com.yunzhi.nanyang.common.alipay;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.domain.AlipayTradeAppPayModel;
import com.alipay.api.domain.AlipayTradeRefundModel;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.nanyang.entity.SysAliPayConfig;
import com.yunzhi.nanyang.service.ISysAliPayConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 主要用来实现支付等相关功能
 */
@Component
public class AlipayService {

    ISysAliPayConfigService iSysAliPayConfigService;
    AlipayFactory alipayFactory = null;
    AlipayFactory.Config config = null;
    AlipayClient alipayClient = null;

    @Autowired
    public AlipayService(ISysAliPayConfigService iSysAliPayConfigService) {
        this.iSysAliPayConfigService = iSysAliPayConfigService;

        try {
            config = getConfig();
            alipayFactory = new AlipayFactory(config);
            alipayClient = alipayFactory.getAlipayClient();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 支付下单
     * @param model
     * @param client
     * @return
     * @throws AlipayApiException
     */
    public AlipayTradeAppPayResponse createOrder(AlipayTradeAppPayModel model, String client) throws AlipayApiException {
        if (null == alipayClient) {
            try {
                config = getConfig();
                alipayFactory = new AlipayFactory(config);
                alipayClient = alipayFactory.getAlipayClient();
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        String notifyUrl = String.format("%s/alipay/%s/order", config.getNotifyUrl(), client);

        //实例化具体API对应的request类,类名称和接口名称对应,当前调用接口名称：alipay.trade.app.pay
        AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
        request.setBizModel (model);
        request.setNotifyUrl(notifyUrl);
        return alipayClient.sdkExecute( request );
    }

    /**
     * 退款
     * @param model
     * @param client
     * @return
     * @throws AlipayApiException
     */
    public AlipayTradeRefundResponse refund(AlipayTradeRefundModel model, String client) throws AlipayApiException {
        if (null == alipayClient) {
            try {
                config = getConfig();
                alipayFactory = new AlipayFactory(config);
                alipayClient = alipayFactory.getAlipayClient();
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        String notifyUrl = String.format("%s/alipay/%s/refund", config.getNotifyUrl(), client);

        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
        request.setBizModel(model);
        request.setNotifyUrl(notifyUrl);
        return alipayClient.execute(request);
    }

    /**
     * 获取 支付宝配置
     * @return
     */
    public boolean rsaCheckV1(Map<String, String> params) throws AlipayApiException {
        if (null == config) return false;

        return AlipaySignature.rsaCheckV1 (params, config.getAlipayPublicCertPath(), config.getCharset(),config.getSignType());
    }

    private AlipayFactory.Config getConfig() throws Exception {
        SysAliPayConfig sysAliPayConfig = iSysAliPayConfigService.getOne(new QueryWrapper<SysAliPayConfig>().eq("status", 1).last("limit 1"));
        if (null == sysAliPayConfig) {
            throw new Exception("未找到支付宝支付相关配置");
        }

        AlipayFactory.Config config = new AlipayFactory.Config();
        config.setAppid(sysAliPayConfig.getAppid());
        config.setPrivateKey(sysAliPayConfig.getPrivateKey());
        config.setCertPath(sysAliPayConfig.getCertPath());
        config.setAlipayPublicCertPath(sysAliPayConfig.getAlipayPublicCertPath());
        config.setRootCertPath(sysAliPayConfig.getRootCertPath());
        config.setServerUrl(sysAliPayConfig.getServerUrl());
        config.setNotifyUrl(sysAliPayConfig.getNotifyUrl());
        config.setAuthUrl(sysAliPayConfig.getAuthUrl());
        config.setAuthCallback(sysAliPayConfig.getAuthCallback());
        config.setFormat("json");
        config.setCharset("UTF-8");
        config.setSignType("RSA2");

        return config;
    }
}
