package com.yunzhi.nanyang.config;

import cn.dev33.satoken.context.SaHolder;
import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.filter.SaServletFilter;
import cn.dev33.satoken.interceptor.SaRouteInterceptor;
import cn.dev33.satoken.jwt.StpLogicJwtForStateless;
import cn.dev33.satoken.router.SaHttpMethod;
import cn.dev33.satoken.router.SaRouter;
import cn.dev33.satoken.stp.StpLogic;
import cn.dev33.satoken.stp.StpUtil;
import com.alibaba.fastjson.JSONObject;
import com.yunzhi.nanyang.common.ResponseBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.util.ArrayList;
import java.util.List;

@Configuration
public class SaTokenConfigure {
//public class SaTokenConfigure implements WebMvcConfigurer {

    @Autowired
    FilterConfig filterConfig;

    @Bean
    public SaServletFilter getSaServletFilter() {
        SaServletFilter saServletFilter = new SaServletFilter();
        saServletFilter.addInclude("/**");
        saServletFilter.addExclude(filterConfig.getAnnList().toArray(new String[0]));
        saServletFilter.setAuth(auth -> {
            // OPTIONS 请求不校验, 主要是为了解决跨域问题
            SaRouter.notMatch(SaHttpMethod.OPTIONS).check(r -> StpUtil.checkLogin());
        });

        saServletFilter.setError(err -> {
            SaHolder.getResponse().setHeader("Content-Type", "application/json;charset=UTF-8");
            SaHolder.getResponse().setStatus(200);

            ResponseBean responseBean = new ResponseBean<>();
            responseBean.setMessage(err.getMessage());
            if (err instanceof NotLoginException) {
                responseBean.setCode(ResponseBean.ERROR_AUTH_FAIL);
            } else {
                responseBean.setCode(ResponseBean.ERROR_UNAVAILABLE);
            }

            String jsonStr = JSONObject.toJSONString(responseBean);
            return jsonStr;
        });

        return saServletFilter;
    }

//    // 注册拦截器
//    @Override
//    public void addInterceptors(InterceptorRegistry registry) {
//
//        List<String> anonList = new ArrayList<>();
//        anonList.add("/swagger-ui/**");
//        anonList.add("/swagger-resources/**");
//        anonList.add("/v2/**");
//        anonList.add("/wxpay/notify/**");
//        anonList.add("/alipay/notify/**");
//        anonList.add("/**/sms-captcha");
//        anonList.add("/**/**/sms-captcha");
//        anonList.add("/**/preload");
//        anonList.add("/**/login");
//        anonList.add("/**/**/login");
//
//        // 注册Sa-Token的路由拦截器
//        registry.addInterceptor(new SaRouteInterceptor())
//                .addPathPatterns("/**")
//                .excludePathPatterns(anonList.toArray(new String[0]));
//    }



    // Sa-Token 整合 jwt (Style模式)
    @Bean
    public StpLogic getStpLogicJwt() {
        return new StpLogicJwtForStateless();
    }
}
