package com.yunzhi.nanyang.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.service.ISysUserService;
import com.yunzhi.nanyang.service.ITaPersonService;
import com.yunzhi.nanyang.vo.LoginParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;


@Api(tags = "手机端登入/登出")
@RestController
@RequestMapping("/{client}/{clientId}")
public class MobileController extends BaseController {

    @Autowired
    public ITaPersonService iTaPersonService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public SMSCaptcha smsCaptcha;

    @PostMapping("/login")
    @ApiOperation(value="app登录", notes = "app登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("客户端") @PathVariable String client,
                              @ApiParam("客户端") @PathVariable String clientId,
                              @ApiParam("登录参数") @RequestBody LoginParam loginParam) throws Exception {

        checkMClient(client);

        String phone = loginParam.getUserName();
        if (StringUtils.isEmpty(phone)) {
            return ResponseBean.error("请设置登录手机号");
        }

        String captchaCode = loginParam.getPassword();
        if (StringUtils.isEmpty(captchaCode)) {
            return ResponseBean.error("请设置登录验证码");
        }

        boolean isOk = smsCaptcha.validate(phone, captchaCode);
        if (!isOk) {
            return ResponseBean.error("验证码不正确或者已过期");
        }

        // 如果是农机手
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            SysUser sysUser = iSysUserService.getExistBy("phone", phone, false, true);
            if (null == sysUser) {
                return ResponseBean.error("未找到对应手机号的农机手信息");
            }
        }

        TaPerson taPerson = iTaPersonService.getByPhone(clientId, phone);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setPhone(phone);
            taPerson.setIdentity(clientId);
            taPerson.setStatus(Constants.STATUS_NORMAL);
            taPerson.setCreateDate(LocalDateTime.now());
            iTaPersonService.save(taPerson);
        }

        // 如果是农机手
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            iTaPersonService.tryBindUser(taPerson);
        }

        StpUtil.login(taPerson.getPersonId(), clientId);
        Map<String, Object> res = new HashMap<>();
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());

        return ResponseBean.success(res);
    }
}
