package com.yunzhi.nanyang.controller;

import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipaySignature;
import com.github.binarywang.wxpay.bean.notify.*;
import com.github.binarywang.wxpay.service.WxPayService;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.WxUtils;
import com.yunzhi.nanyang.common.alipay.AlipayFactory;
import com.yunzhi.nanyang.common.alipay.AlipayService;
import com.yunzhi.nanyang.service.ITaPayService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

@Api(tags = "支付回调")
@RestController
@RequestMapping("/")
public class PayNotifyController extends BaseController {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    AlipayService alipayService;

    @Autowired
    ITaPayService orderPayService;

    @ResponseBody
    @RequestMapping("/wxpay/notify/{client}/order")
    public String wxNotify(@PathVariable String client,
                            @RequestBody String jsonData) {
        WxPayService payService = null;
        if ("ios".equals(client) || "android".equals(client)) {
            payService = wxUtils.getAndroidPayService();
        } else {
            payService = wxUtils.getMiniPayService();
        }

        WxPayOrderNotifyV3Result result = null;
        try {
            result = payService.parseOrderNotifyV3Result(jsonData, null);
            orderPayService.notifyOrder(result.getResult());
            return WxPayNotifyResponse.success("OK");
        } catch (Exception e) {
            e.printStackTrace();
            return WxPayNotifyResponse.fail(e.getMessage());
        }
    }

    @ResponseBody
    @RequestMapping("/wxpay/notify/{client}/refund")
    public String wxRefundNotify(@PathVariable String client,
                               @RequestBody String jsonData) {
        WxPayService payService = null;
        if ("ios".equals(client) || "android".equals(client)) {
            payService = wxUtils.getAndroidPayService();
        } else {
            payService = wxUtils.getMiniPayService();
        }

        WxPayRefundNotifyV3Result result = null;
        try {
            result = payService.parseRefundNotifyV3Result(jsonData, null);
            orderPayService.refundNotifyOrder(result.getResult());
            return WxPayNotifyResponse.success("OK");
        } catch (Exception e) {
            e.printStackTrace();
            return WxPayNotifyResponse.fail(e.getMessage());
        }

    }


    @ResponseBody
    @RequestMapping("/alipay/notify/{client}/order")
    public String aliNotify(@PathVariable String client,
                            HttpServletRequest request) {
        Map<String, String> params = getRequestParams(request);

        //切记alipaypublickey是支付宝的公钥，请去open.alipay.com对应应用下查看。
        //boolean AlipaySignature.rsaCheckV1(Map<String, String> params, String publicKey, String charset, String sign_type)
        try {
            boolean isOk = alipayService.rsaCheckV1(params);
            if (!isOk) return "failure";

            orderPayService.notifyOrder(params);
            return "success";
        } catch (Exception e) {
            e.printStackTrace();
            return "failure";
        }
    }


    @ResponseBody
    @RequestMapping("/alipay/notify/{client}/refund")
    public String aliRefundNotify(@PathVariable String client,
                                  HttpServletRequest request) {
        Map<String, String> params = getRequestParams(request);

        try {
            boolean isOk = alipayService.rsaCheckV1(params);
            if (!isOk) return "failure";

            orderPayService.refundNotifyOrder(params);
            return "success";
        } catch (Exception e) {
            e.printStackTrace();
            return "failure";
        }

    }

    private Map<String, String> getRequestParams(HttpServletRequest request) {
        Map<String, String> params = new HashMap<>();
        Map requestParams = request.getParameterMap();
        for(Iterator iter = requestParams.keySet().iterator(); iter.hasNext();){
            String name = (String)iter.next();
            String[] values = (String [])requestParams.get(name);
            String valueStr = "";
            for(int i = 0;i < values.length;i ++ ){
                valueStr =  (i==values.length-1)?valueStr + values [i]:valueStr + values[i] + ",";
            }
            //乱码解决，这段代码在出现乱码时使用。
            //valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
            params.put (name,valueStr);
        }
        return params;
    }

}
