package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.TaMachinery;
import com.yunzhi.nanyang.entity.TaOrg;
import com.yunzhi.nanyang.entity.TaWorkMessage;
import com.yunzhi.nanyang.entity.TdMachineryType;
import com.yunzhi.nanyang.service.*;
import com.yunzhi.nanyang.vo.ChartParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;


@Api(tags = "数据大屏")
@RestController
@RequestMapping("/admin/screen-data")
public class ScreenDataController extends BaseController {

    @Autowired
    ITaMachineryService iTaMachineryService;

    @Autowired
    ITaOrderService iTaOrderService;

    @Autowired
    ITdMachineryTypeService iTdMachineryTypeService;

    @Autowired
    ITaOrgService iTaOrgService;

    @Autowired
    ITaWorkMessageService iTaWorkMessageService;


    @GetMapping("/basic")
    @ApiOperation(value="基本数据统计", notes = "基本数据统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getBasicStatis() throws Exception {
        // 农机总数
        int totalMachineryNum = iTaMachineryService.countBy("1", 1, true);
        // 农机使用数
        int totalMachineryUsed = 0;
        // 总预约数 == 订单数
        int totalOrderNum = iTaOrderService.countBy("1", 1, true);
        // 总服务数
        int totalServiceNum = 0;

        List<ChartParam> result = new ArrayList<>();
        result.add(new ChartParam("totalMachineryNum", totalMachineryNum));
        result.add(new ChartParam("totalMachineryUsed", totalMachineryUsed));
        result.add(new ChartParam("totalOrderNum", totalOrderNum));
        result.add(new ChartParam("totalServiceNum", totalServiceNum));

        return ResponseBean.success(result);
    }

    @GetMapping("/machinery-type")
    @ApiOperation(value="所有农机类型", notes = "所有农机类型", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryType() throws Exception {
        List<TdMachineryType> result = iTdMachineryTypeService.getTypesOfScreen();
        return ResponseBean.success(result);
    }

    @GetMapping("/machinery-type/machinery")
    @ApiOperation(value="农机类型统计", notes = "农机类型统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryTypeStatis() throws Exception {
        List<ChartParam> result = iTdMachineryTypeService.statisMachineryNum();
        return ResponseBean.success(result);
    }

    @GetMapping("/machinery-type/work-job")
    @ApiOperation(value="农机作业数统计", notes = "农机作业数统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryTypeWorkJobs() throws Exception {
        List<ChartParam> result = iTdMachineryTypeService.statisWorkJobNum();
        return ResponseBean.success(result);
    }


    @GetMapping("/machinery-type/area")
    @ApiOperation(value="农机作业面积统计", notes = "农机作业面积统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryTypeArea() throws Exception {
        List<ChartParam> result = iTdMachineryTypeService.statisArea();
        return ResponseBean.success(result);
    }

    @GetMapping("/org")
    @ApiOperation(value="所有合作社", notes = "所有合作社", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getAllOrgs() throws Exception {
        QueryWrapper<TaOrg> queryWrapper = new QueryWrapper<TaOrg>()
                .gt("status", Constants.STATUS_DELETE);
        List<TaOrg> result = iTaOrgService.list(queryWrapper);
        return ResponseBean.success(result);
    }


    @GetMapping("/machinery/all")
    @ApiOperation(value="依据类型查询农机", notes = "依据类型查询农机", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryOfType(@ApiParam("农机类型") @RequestParam(value = "typeId", required = false) String typeId) throws Exception {
        List<TaMachinery> result = iTaMachineryService.listByTypeWithLoc(typeId);
        return ResponseBean.success(result);
    }

    @GetMapping("/message")
    @ApiOperation(value="获取消息通知", notes = "获取消息通知", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMessage(@ApiParam("数据条数") @RequestParam(value = "limit", defaultValue = "20") Integer limit,
                                   @ApiParam(value = "消息类型, appoint 预约消息, accept 作业消息", allowableValues = "appoint,accept") @RequestParam(value = "messageType", required = false) String messageType) throws Exception {
        List<TaWorkMessage> result = iTaWorkMessageService.listByType(messageType, limit);
        return ResponseBean.success(result);
    }
}
