package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.service.ISysRolePermissionService;
import com.yunzhi.nanyang.service.ISysUserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysRoleService;
import com.yunzhi.nanyang.entity.SysRole;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 角色表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-07
 */

@Api(tags = "角色表")
@RestController
@RequestMapping("/")
public class SysRoleController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysRoleController.class);

    @Autowired
    public ISysRoleService iSysRoleService;

    @Autowired
    public ISysUserRoleService iSysUserRoleService;

    @Autowired
    public ISysRolePermissionService iSysRolePermissionService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/role",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysRoleList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("角色名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<SysRole> pg = new Page<>(pageNum, pageSize);
        String userId = currentUser().getUserId();
        boolean isAdmin = userId.equals(Constants.ADMIN_ID);

        // 管理员可以查询所有角色
        // 普通用户只能查询自己授权的角色
        IPage<SysRole> result = iSysRoleService.getPageBy(pg, isAdmin, userId, name);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysRole 实体对象
     * @return
     */
    @RequestMapping(value="/admin/role",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("role:add")
    public ResponseBean sysRoleAdd(@ApiParam("保存内容") @RequestBody SysRole sysRole) throws Exception{
        sysRole.setRoleId(null);

        if (StringUtils.isEmpty(sysRole.getName())) {
            return ResponseBean.error("角色名称不能为空");
        }

        int cnt = iSysRoleService.countBy("name", sysRole.getName(), false);
        if (cnt > 0) {
            return ResponseBean.error("名称重复");
        }

        if (iSysRoleService.save(sysRole)){
            return ResponseBean.success(sysRole);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/role/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("role:delete")
    public ResponseBean sysRoleDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{

        int cnt = iSysUserRoleService.countBy("role_id", id, false);
        if (cnt > 0) {
            return ResponseBean.error("请先取消用户授权");
        }

        if(iSysRoleService.removeById(id)){

            iSysRolePermissionService.removeBy("role_id", id, false);

            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysRole 实体对象
     * @return
     */
    @RequestMapping(value="/admin/role/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("role:update")
    public ResponseBean sysRoleUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody SysRole sysRole) throws Exception{

        sysRole.setRoleId(id);

        if (StringUtils.isEmpty(sysRole.getName())) {
            return ResponseBean.error("角色名称不能为空");
        }

        SysRole origin = iSysRoleService.getByButNot("name", sysRole.getName(), "role_id", id, false);
        if (origin != null) {
            return ResponseBean.error("名称重复");
        }

        if (iSysRoleService.updateById(sysRole)){
            return ResponseBean.success(iSysRoleService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/role/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("role:view")
    public ResponseBean sysRoleGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iSysRoleService.getById(id));
    }
}
