package com.yunzhi.nanyang.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysRolePermissionService;
import com.yunzhi.nanyang.entity.SysRolePermission;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 角色权限 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-07
 */

@Api(tags = "角色权限")
@RestController
@RequestMapping("/")
public class SysRolePermissionController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysRolePermissionController.class);

    @Autowired
    public ISysRolePermissionService iSysRolePermissionService;


    /**
     * 分页查询列表
     * @param roleId
     * @return
     */
    @RequestMapping(value="/admin/role-permission",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysRolePermissionList(@ApiParam("角色ID") @RequestParam(value ="roleId") String roleId) throws Exception{

//        String userId = currentUser().getUserId();
//        boolean isAdmin = userId.equals(Constants.ADMIN_ID);

        List<SysRolePermission> result = iSysRolePermissionService.listBy(roleId);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysRolePermission 实体对象
     * @return
     */
    @RequestMapping(value="/admin/role/{roleId}/permission",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("role:add")
    public ResponseBean sysRolePermissionAdd(@ApiParam("角色ID") @PathVariable String roleId,
                                             @ApiParam("保存内容") @RequestBody List<SysRolePermission> sysRolePermissionList) throws Exception{

        if (sysRolePermissionList.isEmpty()) {
            return ResponseBean.error("授权内容不能为空");
        }

        // 判断是否有角色操作权限
        // todo

        for (SysRolePermission item : sysRolePermissionList) {
            item.setSerialNo(null);
            item.setRoleId(roleId);

            String permissionId = item.getPermissionId();
            if (StringUtils.isEmpty(roleId) || StringUtils.isEmpty(permissionId)) {
                return ResponseBean.error("存在授权空内容");
            }
        }

        // 先删除原来的
        Map<String, Object> params = new HashMap<>();
        params.put("role_id", roleId);
        iSysRolePermissionService.removeByMap(params);

        if (iSysRolePermissionService.saveBatch(sysRolePermissionList)){
            return ResponseBean.success(sysRolePermissionList);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/role-permission/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("role:delete")
    public ResponseBean sysRolePermissionDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{

        SysRolePermission rolePermission = iSysRolePermissionService.getById(id);
        if (null == rolePermission) {
            return ResponseBean.success("success");
        }

        if (!checkAccess(currentUser().getUserId(), rolePermission.getRoleId(), rolePermission.getPermissionId())) {
            return ResponseBean.error("无权限进行此操作");
        }

        if(iSysRolePermissionService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    private boolean checkAccess(String userId, String roleId, String permissionId) {
        if (Constants.ADMIN_ID.equals(userId)) return true;

        return iSysRolePermissionService.checkAccess(userId, roleId, permissionId);
    }

}
