package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysSettingService;
import com.yunzhi.nanyang.entity.SysSetting;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 系统设置 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "系统设置")
@RestController
@RequestMapping("/")
public class SysSettingController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysSettingController.class);

    @Autowired
    public ISysSettingService iSysSettingService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/setting",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysSettingList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        IPage<SysSetting> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysSetting> queryWrapper = new QueryWrapper<>();
        queryWrapper.lt("is_hidden", Constants.STATUS_NORMAL);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByAsc("sort_no");

        IPage<SysSetting> result = iSysSettingService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysSetting 实体对象
     * @return
     */
    @RequestMapping(value="/admin/setting/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("setting:update")
    public ResponseBean sysSettingUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody SysSetting sysSetting) throws Exception{

        if (iSysSettingService.updateById(sysSetting)){
            return ResponseBean.success(iSysSettingService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

}
