package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.SysRole;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.service.ISysRoleService;
import com.yunzhi.nanyang.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysUserRoleService;
import com.yunzhi.nanyang.entity.SysUserRole;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;

/**
 * <p>
    * 用户角色 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-07
 */

@Api(tags = "用户角色")
@RestController
@RequestMapping("/")
public class SysUserRoleController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysUserRoleController.class);

    @Autowired
    public ISysUserRoleService iSysUserRoleService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ISysRoleService iSysRoleService;

    /**
     * 分页查询列表
     * @param userId
     * @return
     */
    @RequestMapping(value="/admin/user-role",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysUserRoleList(@ApiParam("用户ID") @RequestParam(value ="user_id") String userId) throws Exception{

        List<SysUserRole> result = iSysUserRoleService.listByUser(userId);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysUserRole 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user/{userId}/role",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("user:add")
    public ResponseBean sysUserRoleAdd(@ApiParam("用户ID") @PathVariable String userId,
                                       @ApiParam("保存内容") @RequestBody List<SysUserRole> sysUserRoleList) throws Exception{
        // 不能编辑管理员权限
        if (Constants.ADMIN_ID.equals(userId)) {
            return ResponseBean.error("无操作权限");
        }
        //
        SysUser sysUser = iSysUserService.getExistBy("user_id", userId, false,true);
        if (sysUser == null) {
            return ResponseBean.error("用户不存在");
        }

        // 操作员与待授权人员必须同属一个机构
        String currentUserId = currentUser().getUserId();
        if (!iSysUserService.canAccessData(sysUser.getOrgId(), currentUserId)) {
            return ResponseBean.error("无操作权限");
        }

        if (sysUserRoleList.isEmpty()) {
            return ResponseBean.error("未找到授权内容");
        }

        for (SysUserRole sysUserRole : sysUserRoleList) {

            String roleId = sysUserRole.getRoleId();
            if (StringUtils.isEmpty(roleId) || StringUtils.isEmpty(userId)) {
                return ResponseBean.error("用户或者角色不能为空");
            }

            SysRole sysRole = iSysRoleService.getById(roleId);
            if (sysRole == null) {
                return ResponseBean.error("角色不存在");
            }

            if (iSysUserRoleService.checkExist(roleId, currentUserId) == null) {
                return ResponseBean.error("无操作权限");
            }
        }

        // 先删除所有
        iSysUserRoleService.removeByMap(new HashMap<String, Object>(){{
            put("user_id", userId);
        }});

        if (iSysUserRoleService.saveBatch(sysUserRoleList)){
            return ResponseBean.success(sysUserRoleList);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/user/{userId}/role", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("user:delete")
    public ResponseBean sysUserRoleDelete(@ApiParam("对象ID") @PathVariable String userId) throws Exception{
        // 不能编辑管理员权限
        if (Constants.ADMIN_ID.equals(userId)) {
            return ResponseBean.error("无操作权限");
        }
        //
        SysUser sysUser = iSysUserService.getExistBy("user_id", userId, false,true);
        if (sysUser == null) {
            return ResponseBean.error("用户不存在");
        }

        // 操作员与待授权人员必须同属一个机构
        String currentUserId = currentUser().getUserId();
        if (!iSysUserService.canAccessData(sysUser.getOrgId(), currentUserId)) {
            return ResponseBean.error("无操作权限");
        }

        iSysUserRoleService.removeByMap(new HashMap<String, Object>(){{
            put("user_id", userId);
        }});

        return ResponseBean.success("success");
    }

}
