package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysWxPayConfigService;
import com.yunzhi.nanyang.entity.SysWxPayConfig;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 微信支付配置 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-06-22
 */

@Api(tags = "微信支付配置")
@RestController
@RequestMapping("/")
public class SysWxPayConfigController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysWxPayConfigController.class);

    @Autowired
    public ISysWxPayConfigService iSysWxPayConfigService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/sysWxPayConfig",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysWxPayConfigList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

		    IPage<SysWxPayConfig> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<SysWxPayConfig> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<SysWxPayConfig> result = iSysWxPayConfigService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysWxPayConfig 实体对象
     * @return
     */
    @RequestMapping(value="/sysWxPayConfig",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean sysWxPayConfigAdd(@ApiParam("保存内容") @RequestBody SysWxPayConfig sysWxPayConfig) throws Exception{

        if (iSysWxPayConfigService.save(sysWxPayConfig)){
            return ResponseBean.success(sysWxPayConfig);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysWxPayConfig/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean sysWxPayConfigDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iSysWxPayConfigService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysWxPayConfig 实体对象
     * @return
     */
    @RequestMapping(value="/sysWxPayConfig/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean sysWxPayConfigUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody SysWxPayConfig sysWxPayConfig) throws Exception{

        if (iSysWxPayConfigService.updateById(sysWxPayConfig)){
            return ResponseBean.success(iSysWxPayConfigService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysWxPayConfig/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysWxPayConfigGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iSysWxPayConfigService.getById(id));
    }
}
