package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaAccount;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.service.ITaAccountService;
import com.yunzhi.nanyang.vo.AccountRecord;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaAccountLogService;
import com.yunzhi.nanyang.entity.TaAccountLog;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 流水表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "流水表")
@RestController
@RequestMapping("/")
public class TaAccountLogController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaAccountLogController.class);

    @Autowired
    public ITaAccountLogService iTaAccountLogService;

    @Autowired
    public ITaAccountService iTaAccountService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/account-log",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("fi-system:view")
    public ResponseBean taAccountLogList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @ApiParam(value = "合作社ID") @RequestParam(value ="orgId", required = false) String orgId,
                                         @ApiParam(value = "起始时间", example = "2022-03-01") @RequestParam(value ="start", required = false) String start,
                                         @ApiParam(value = "结束时间", example = "2022-03-31") @RequestParam(value ="end", required = false) String end,
                                         @ApiParam(value = "订单号") @RequestParam(value ="orderNo", required = false) String orderNo,
                                         @ApiParam(value = "收支类型") @RequestParam(value ="feeType", required = false) Integer feeType,
                                         @ApiParam(value = "消费类型") @RequestParam(value ="chargeType", required = false) String chargeType) throws Exception{

        checkOrgAccess(orgId);

        LocalDateTime startDate = DateUtils.getDayStart(start);
        LocalDateTime endDate = DateUtils.getDayEnd(end);


        IPage<AccountRecord> pg = new Page<>(pageNum, pageSize);

        IPage<AccountRecord> result = iTaAccountLogService.getPageBy(pg, orgId, startDate, endDate, orderNo, feeType, chargeType);
        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/worker/account-log",method= RequestMethod.GET)
    @ApiOperation(value="我的钱包明细", notes = "我的钱包明细", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taAccountList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
        TaPerson taPerson = currentPerson();
        SysUser sysUser = checkWorker(taPerson);
        if (null == sysUser.getIsOrgManager() || !sysUser.getIsOrgManager()) {
            return ResponseBean.error("");
        }

        TaAccount taAccount = iTaAccountService.getUserAccount(sysUser.getOrgId());

        IPage<TaAccountLog> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaAccountLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", taAccount.getAccountId());
        queryWrapper.orderByDesc("create_date");

        IPage<TaAccountLog> result = iTaAccountLogService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }
}
