package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.TaPerson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaAddressService;
import com.yunzhi.nanyang.entity.TaAddress;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 地址 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "地址")
@RestController
@RequestMapping("/")
public class TaAddressController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaAddressController.class);

    @Autowired
    public ITaAddressService iTaAddressService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/address",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taAddressList(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        TaPerson taPerson = currentPerson();

        IPage<TaAddress> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaAddress> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("owner_type", clientId);
        queryWrapper.eq("owner_id", taPerson.getPersonId());
        queryWrapper.orderByAsc("sort_no");

        IPage<TaAddress> result = iTaAddressService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taAddress 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/address",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taAddressAdd(@ApiParam("客户端") @PathVariable String client,
                                     @ApiParam("客户端ID") @PathVariable String clientId,
                                     @ApiParam("保存内容") @RequestBody TaAddress taAddress) throws Exception{
        TaPerson taPerson = currentPerson();

        taAddress.setAddressId(null);
        taAddress.setOwnerId(taPerson.getPersonId());
        taAddress.setOwnerType(clientId);
        taAddress.setStatus(Constants.STATUS_NORMAL);

        if (iTaAddressService.save(taAddress)){
            return ResponseBean.success(taAddress);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/address/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taAddressDelete(@ApiParam("客户端") @PathVariable String client,
                                        @ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaPerson current = currentPerson();
        TaAddress taAddress = iTaAddressService.getById(id);

        if (null == taAddress) {
            return ResponseBean.error("未找到地址信息");
        }

        if (!clientId.equals(taAddress.getOwnerType()) || !current.getPersonId().equals(taAddress.getOwnerId())) {
            return ResponseBean.error("无操作权限");
        }

        if(iTaAddressService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taAddress 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/address/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taAddressUpdate(@ApiParam("客户端") @PathVariable String client,
                                        @ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaAddress taAddress) throws Exception{

        TaPerson current = currentPerson();
        TaAddress origin = iTaAddressService.getById(id);

        if (null == origin) {
            return ResponseBean.error("未找到地址信息");
        }

        if (!clientId.equals(origin.getOwnerType()) || !current.getPersonId().equals(origin.getOwnerId())) {
            return ResponseBean.error("无操作权限");
        }

        taAddress.setAddressId(id);

        if (iTaAddressService.updateById(taAddress)){
            return ResponseBean.success(taAddress);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 设置默认地址
     * @param id  实体ID
     * @param taAddress 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/address/{id}/default",method= RequestMethod.PUT)
    @ApiOperation(value="设置默认地址", notes = "设置默认地址", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean updateDefault(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("对象ID") @PathVariable String id) throws Exception{

        TaPerson current = currentPerson();
        TaAddress origin = iTaAddressService.getById(id);

        if (null == origin) {
            return ResponseBean.error("未找到地址信息");
        }

        if (!clientId.equals(origin.getOwnerType()) || !current.getPersonId().equals(origin.getOwnerId())) {
            return ResponseBean.error("无操作权限");
        }

        if (iTaAddressService.updateDefault(origin)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }
}
