package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.TaPerson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaBankCardService;
import com.yunzhi.nanyang.entity.TaBankCard;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 银行卡 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-04-08
 */

@Api(tags = "银行卡")
@RestController
@RequestMapping("/")
public class TaBankCardController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaBankCardController.class);

    @Autowired
    public ITaBankCardService iTaBankCardService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/worker/bankcard",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taBankCardList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
        TaPerson taPerson = currentPerson();
        checkWorker(taPerson);

        IPage<TaBankCard> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaBankCard> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.eq("person_id", taPerson.getPersonId());
        queryWrapper.orderByDesc("create_date");

        IPage<TaBankCard> result = iTaBankCardService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taBankCard 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/bankcard",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taBankCardAdd(@ApiParam("保存内容") @RequestBody TaBankCard taBankCard) throws Exception{
        TaPerson taPerson = currentPerson();
        checkWorker(taPerson);

        taBankCard.setPersonId(taPerson.getPersonId());
        if (iTaBankCardService.save(taBankCard)){
            return ResponseBean.success(taBankCard);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/worker/bankcard/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taBankCardDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaBankCard bankCard = validate(id);

        if(iTaBankCardService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taBankCard 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/bankcard/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taBankCardUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaBankCard taBankCard) throws Exception{
        TaBankCard bankCard = validate(id);
        taBankCard.setCardId(id);
        taBankCard.setPersonId(bankCard.getPersonId());

        if (iTaBankCardService.updateById(taBankCard)){
            return ResponseBean.success(iTaBankCardService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/worker/bankcard/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taBankCardGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(validate(id));
    }

    private TaBankCard validate(String cardId) throws Exception {
        TaPerson taPerson = currentPerson();
        checkWorker(taPerson);

        TaBankCard bankCard = iTaBankCardService.getExistBy("card_id", cardId, false, true);
        if (null == bankCard) {
            throw new Exception("银行卡不存在");
        }

        if (!taPerson.getPersonId().equals(bankCard.getPersonId())) {
            throw new Exception("无权操作");
        }

        return bankCard;
    }
}
