package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaBannerService;
import com.yunzhi.nanyang.entity.TaBanner;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
    * 轮播图表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "轮播图表")
@RestController
@RequestMapping("/")
public class TaBannerController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaBannerController.class);

    @Autowired
    public ITaBannerService iTaBannerService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/banner",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taBannerList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("名称") @RequestParam(value ="title", required = false) String title,
                                     @ApiParam("类型") @RequestParam(value ="position", required = false) String position) throws Exception{

        IPage<TaBanner> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaBanner> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(title), "title", "%" + title + "%");
        queryWrapper.eq(!StringUtils.isEmpty(position), "position", position);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<TaBanner> result = iTaBannerService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param client
     * @param clientId
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/banner",method= RequestMethod.GET)
    @ApiOperation(value="移动端列表", notes = "移动端列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("客户端") @PathVariable String client,
                                @ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("类型") @RequestParam(value ="position", required = false) String position) throws Exception{

        QueryWrapper<TaBanner> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(!StringUtils.isEmpty(position), "position", position);
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");

        List<TaBanner> result = iTaBannerService.list(queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taBanner 实体对象
     * @return
     */
    @RequestMapping(value="/admin/banner",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("banner:add")
    public ResponseBean taBannerAdd(@ApiParam("保存内容") @RequestBody TaBanner taBanner) throws Exception{
        if (StringUtils.isEmpty(taBanner.getTitle())) {
            return ResponseBean.error("标题不能为空");
        }
        if (StringUtils.isEmpty(taBanner.getThumb())) {
            return ResponseBean.error("图片不能为空");
        }
        if (StringUtils.isEmpty(taBanner.getPosition())) {
            return ResponseBean.error("位置不能为空");
        }

        taBanner.setBannerId(null);

        if (iTaBannerService.save(taBanner)){
            return ResponseBean.success(taBanner);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/banner/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("banner:delete")
    public ResponseBean taBannerDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaBannerService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taBanner 实体对象
     * @return
     */
    @RequestMapping(value="/admin/banner/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("banner:update")
    public ResponseBean taBannerUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaBanner taBanner) throws Exception{
        taBanner.setBannerId(id);

        if (StringUtils.isEmpty(taBanner.getTitle())) {
            return ResponseBean.error("标题不能为空");
        }
        if (StringUtils.isEmpty(taBanner.getThumb())) {
            return ResponseBean.error("图片不能为空");
        }
        if (StringUtils.isEmpty(taBanner.getPosition())) {
            return ResponseBean.error("位置不能为空");
        }

        if (iTaBannerService.updateById(taBanner)){
            return ResponseBean.success(taBanner);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

}
