package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaWorkJob;
import com.yunzhi.nanyang.event.WorkDispatchEvent;
import com.yunzhi.nanyang.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.entity.TaDispatch;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 调度表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "调度表")
@RestController
@RequestMapping("/")
public class TaDispatchController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaDispatchController.class);

    @Autowired
    ITaOrgService iTaOrgService;

    @Autowired
    public ITaDispatchService iTaDispatchService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaMachineryService iTaMachineryService;

    @Autowired
    public ITdMachineryTypeService iTdMachineryTypeService;

    @Autowired
    public ITaWorkJobService iTaWorkJobService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    ApplicationEventPublisher applicationEventPublisher;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/dispatch",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taDispatchList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("机构") @RequestParam(value ="orgId", required = false) String orgId,
                                       @ApiParam("农机") @RequestParam(value ="machineryId", required = false) String machineryId,
                                       @ApiParam("农机类型") @RequestParam(value ="typeId", required = false) String typeId,
                                       @ApiParam(value = "预约开始", example = "2022-03-01") @RequestParam(value ="startDate",required = false) String startDate,
                                       @ApiParam(value = "预约结束", example = "2022-03-31") @RequestParam(value ="endDate",required = false) String endDate,
                                       @ApiParam("手机号") @RequestParam(value ="phone",required = false) String phone,
                                       @ApiParam("状态") @RequestParam(value ="status", required = false) String status,
                                       @ApiParam("状态") @RequestParam(value ="isWarn", defaultValue = "false") Boolean isWarn) throws Exception{

        checkOrgAccess(orgId);

        IPage<TaOrder> pg = new Page<>(pageNum, pageSize);

        LocalDateTime start = DateUtils.getDayStart(startDate);
        LocalDateTime end = DateUtils.getDayEnd(endDate);

        IPage<TaOrder> result = iTaDispatchService.getPageBy(pg, orgId, typeId, machineryId, start, end, phone, status, isWarn);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taDispatch 实体对象
     * @return
     */
    @RequestMapping(value="/admin/dispatch",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("orderjob:add")
    public ResponseBean taDispatchAdd(@ApiParam("保存内容") @RequestBody TaDispatch taDispatch) throws Exception{
        taDispatch.setDispatchId(null);

        checkOrgAccess(taDispatch.getOrgId());

        if (StringUtils.isEmpty(taDispatch.getOrderId()) || StringUtils.isEmpty(taDispatch.getOrderId())) {
            return ResponseBean.error("订单或者接单机构为空");
        }
        TaOrder taOrder = iTaOrderService.getExistBy("order_id", taDispatch.getOrderId(), false, true);
        if (null == taOrder) {
            return ResponseBean.error("验证订单信息异常");
        }

        taDispatch.setOrgName(iTaOrgService.getById(taDispatch.getOrgId()).getName());

        if (!StringUtils.isEmpty(taDispatch.getMachineryId())) {
            taDispatch.setMachineryName(iTaMachineryService.getById(taDispatch.getMachineryId()).getName());
        }

        if (!StringUtils.isEmpty(taDispatch.getTypeId())) {
            taDispatch.setTypeName(iTdMachineryTypeService.getById(taDispatch.getTypeId()).getName());
        }

        if (!StringUtils.isEmpty(taDispatch.getWorkerId())) {
            taDispatch.setWorkerName(iSysUserService.getById(taDispatch.getWorkerId()).getUserName());
        }

        taDispatch.setStatus(Constants.STATUS_NORMAL);

        TaWorkJob taWorkJob = iTaDispatchService.createNew(taDispatch, taOrder);
        if (null != taWorkJob){
            // 发送通知
            applicationEventPublisher.publishEvent(new WorkDispatchEvent(this, taWorkJob, taOrder));
            return ResponseBean.success(taDispatch);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/dispatch/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("orderjob:delete")
    public ResponseBean taDispatchDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaDispatch taDispatch = iTaDispatchService.getById(id);

        checkOrgAccess(taDispatch.getOrgId());

        TaWorkJob taWorkJob = iTaWorkJobService.getExistBy("dispatch_id", id, false, true);
        if (null == taWorkJob) {
            return ResponseBean.error("校验作业信息失败");
        }

//        if (Constants.WORK_READY != taWorkJob.getStatus()) {
//            return ResponseBean.error("作业已开始，不能删除");
//        }

        // 先清除原来的作业信息
        iTaWorkJobService.removeLogicById(taWorkJob.getJobId());

        if(iTaDispatchService.removeLogicById(id)){
            // 更新订单状态
            iTaOrderService.updateStatus(taDispatch.getOrderId(), "dispatch_status", Constants.DISPATCH_READY);
            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taDispatch 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taDispatch/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taDispatchUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaDispatch taDispatch) throws Exception{
//
//        if (iTaDispatchService.updateById(taDispatch)){
//            return ResponseBean.success(iTaDispatchService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/dispatch/{id}",method= RequestMethod.GET)
    @ApiOperation(value="调度详情", notes = "调度详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetail(@ApiParam("调度ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaDispatchService.getExistBy("dispatch_id", id, false, true));
    }

    /**
     * 根据id查询对象
     * @param orderId  实体ID
     */
    @RequestMapping(value="/admin/order/{orderId}/dispatch",method= RequestMethod.GET)
    @ApiOperation(value="调度详情", notes = "调度详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taDispatchGet(@ApiParam("订单ID") @PathVariable String orderId) throws Exception{
        TaDispatch taDispatch = iTaDispatchService.getExistBy("order_id", orderId, false, true);
        return ResponseBean.success(taDispatch);
    }
}
