package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.service.ITaOrderService;
import com.yunzhi.nanyang.service.ITaPersonService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaEvaluationService;
import com.yunzhi.nanyang.entity.TaEvaluation;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 评价表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "评价表")
@RestController
@RequestMapping("/")
public class TaEvaluationController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaEvaluationController.class);

    @Autowired
    public ITaEvaluationService iTaEvaluationService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaPersonService iTaPersonService;


//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/taEvaluation",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taEvaluationList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//		    IPage<TaEvaluation> pg = new Page<>(pageNum, pageSize);
//            QueryWrapper<TaEvaluation> queryWrapper = new QueryWrapper<>();
//            queryWrapper.orderByDesc("create_date");
//
//            IPage<TaEvaluation> result = iTaEvaluationService.page(pg, queryWrapper);
//            return ResponseBean.success(result);
//    }

    /**
     * 保存对象
     * @param taEvaluation 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/evaluation",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taEvaluationAdd(@ApiParam("客户端") @PathVariable String client,
                                        @ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("保存内容") @RequestBody TaEvaluation taEvaluation) throws Exception{

        if (StringUtils.isEmpty(taEvaluation.getOrderId())) {
            return ResponseBean.error("订单ID为空");
        }

        TaOrder taOrder = iTaOrderService.getExistBy("order_id", taEvaluation.getOrderId(), false, true);
        if (taOrder == null) {
            return ResponseBean.error("找不到订单");
        }

        String personId = currentPerson().getPersonId();
        if (!taOrder.getPersonId().equals(personId)) {
            return ResponseBean.error("非法操作");
        }

//        if (Constants.PAY_PAID != taOrder.getPayStatus() || Constants.WORK_DONE != taOrder.getWorkStatus()) {
//            return ResponseBean.error("订单未支付或者未完成");
//        }

        taEvaluation.setPersonId(personId);

        if (iTaEvaluationService.save(taEvaluation)){

            iTaOrderService.updateStatus(taOrder.getOrderId(), "is_evaluated", 1);

            return ResponseBean.success(taEvaluation);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taEvaluation/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taEvaluationDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaEvaluationService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taEvaluation 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taEvaluation/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taEvaluationUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaEvaluation taEvaluation) throws Exception{
//
//        if (iTaEvaluationService.updateById(taEvaluation)){
//            return ResponseBean.success(iTaEvaluationService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/order/{orderId}/evaluation",method= RequestMethod.GET)
    @ApiOperation(value="依据订单查询", notes = "依据订单查询", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taEvaluationGet(@ApiParam("订单ID") @PathVariable String orderId) throws Exception{
        TaEvaluation taEvaluation  = iTaEvaluationService.getExistBy("order_id", orderId, false, false);
        if (null == taEvaluation) {
            return ResponseBean.error("未找到评价内容");
        }

        TaPerson taPerson = iTaPersonService.getById(taEvaluation.getPersonId());
        if (taPerson != null) {
            taEvaluation.setPersonName(taPerson.getNickName());
        }

        return ResponseBean.success(taEvaluation);
    }
}
