package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaExtendContentService;
import com.yunzhi.nanyang.entity.TaExtendContent;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 拓展内容表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "拓展内容表")
@RestController
@RequestMapping("/")
public class TaExtendContentController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaExtendContentController.class);

    @Autowired
    public ITaExtendContentService iTaExtendContentService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/extend-content",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taExtendContentList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                            @ApiParam("资源分类") @RequestParam(value ="targetType") String targetType,
                                            @ApiParam("资源ID") @RequestParam(value ="targetId") String targetId) throws Exception{

        IPage<TaExtendContent> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaExtendContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("target_type", targetType);
        queryWrapper.eq("target_id", targetId);
        queryWrapper.orderByAsc("sort");

        IPage<TaExtendContent> result = iTaExtendContentService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taExtendContent 实体对象
     * @return
     */
    @RequestMapping(value="/admin/extend-content",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taExtendContentAdd(@ApiParam("保存内容") @RequestBody TaExtendContent taExtendContent) throws Exception{

        if (StringUtils.isEmpty(taExtendContent.getTargetId()) || StringUtils.isEmpty(taExtendContent.getTargetType())) {
            return ResponseBean.error("资源不能为空");
        }

        if (StringUtils.isEmpty(taExtendContent.getContent())) {
            return ResponseBean.error("内容不能为空");
        }

        taExtendContent.setExtId(null);

        if (iTaExtendContentService.save(taExtendContent)){
            return ResponseBean.success(taExtendContent);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/extend-content/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taExtendContentDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaExtendContentService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taExtendContent 实体对象
     * @return
     */
    @RequestMapping(value="/admin/extend-content/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taExtendContentUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaExtendContent taExtendContent) throws Exception{

        if (StringUtils.isEmpty(taExtendContent.getTargetId()) || StringUtils.isEmpty(taExtendContent.getTargetType())) {
            return ResponseBean.error("资源不能为空");
        }

        if (StringUtils.isEmpty(taExtendContent.getContent())) {
            return ResponseBean.error("内容不能为空");
        }

        taExtendContent.setExtId(id);

        if (iTaExtendContentService.updateById(taExtendContent)){
            return ResponseBean.success(taExtendContent);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/extend-content/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taExtendContentGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaExtendContentService.getById(id));
    }
}
