package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaFeedbackService;
import com.yunzhi.nanyang.entity.TaFeedback;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 意见反馈 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "意见反馈")
@RestController
@RequestMapping("/")
public class TaFeedbackController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaFeedbackController.class);

    @Autowired
    public ITaFeedbackService iTaFeedbackService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/feedback",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taFeedbackList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("姓名") @RequestParam(value ="nickName", required = false) String nickName,
                                       @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone) throws Exception{

        IPage<TaFeedback> pg = new Page<>(pageNum, pageSize);
        IPage<TaFeedback> result = iTaFeedbackService.getPageBy(pg, nickName, phone);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taFeedback 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/feedback",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taFeedbackAdd(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("保存内容") @RequestBody TaFeedback taFeedback) throws Exception{

        taFeedback.setFromPerson(currentPerson().getPersonId());
        taFeedback.setFromPort(clientId);
        taFeedback.setStatus(Constants.STATUS_READY);

        if (iTaFeedbackService.save(taFeedback)){
            return ResponseBean.success(taFeedback);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taFeedback 实体对象
     * @return
     */
    @RequestMapping(value="/admin/feedback/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("feedback:update")
    public ResponseBean taFeedbackUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaFeedback taFeedback) throws Exception{
        if (iTaFeedbackService.updateById(taFeedback)){
            return ResponseBean.success(iTaFeedbackService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/feedback/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taFeedbackGet(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaFeedbackService.getById(id));
    }
}
