package com.yunzhi.nanyang.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.service.*;
import com.yunzhi.nanyang.vo.DeviceSummary;
import com.yunzhi.nanyang.vo.MachineSummary;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 农机表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "农机表")
@RestController
@RequestMapping("/")
public class TaMachineryController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMachineryController.class);

    @Autowired
    public ITaMachineryService iTaMachineryService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ITaOrgService iTaOrgService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITdDeviceService iTdDeviceService;

    @Autowired
    public ITdHatcDeviceService iTdHatcDeviceService;

    @Autowired
    public HttpUtils httpUtils;

    @Value("${hatc.apiServer}")
    private String hatcApiServer;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/machinery",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMachineryList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                        @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                        @ApiParam("名称") @RequestParam(value ="name", required = false) String name,
                                        @ApiParam("类型") @RequestParam(value ="typeId", required = false) String typeId,
                                        @ApiParam("机构ID") @RequestParam(value ="orgId", required = false) String orgId,
                                        @ApiParam("状态") @RequestParam(value ="status", required = false) Integer status) throws Exception{
        checkOrgAccess(orgId);

        IPage<TaMachinery> pg = new Page<>(pageNum, pageSize);

        IPage<TaMachinery> result = iTaMachineryService.getPageBy(pg, name, typeId, orgId, status);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/machinery",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getSummaryList(@ApiParam("客户端") @PathVariable String client,
                                       @ApiParam("客户端ID") @PathVariable String clientId,
                                       @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("当前位置") @RequestParam(value ="location") String location,
                                       @ApiParam("分类") @RequestParam(value = "typeId", required = false) String typeId,
                                       @ApiParam("搜索关键词") @RequestParam(value = "q", required = false) String q) throws Exception{

        String orgId = null;
        TaPerson current = currentPerson();
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            // 农机手查询机器列表, 只能查询自己机构的机器
            if (StringUtils.isEmpty(current.getUserId())) {
                return ResponseBean.error("未找到绑定农机手信息");
            }

            SysUser user = iSysUserService.getExistBy("user_id", current.getUserId(), false, true);
            if (user == null) {
                return ResponseBean.error("未找到绑定农机手信息");
            }
            orgId = user.getOrgId();
        }

        IPage<MachineSummary> pg = new Page<>(pageNum, pageSize);

        IPage<MachineSummary> result = iTaMachineryService.getSummaryList(pg, location, typeId, orgId, q, clientId);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taMachinery 实体对象
     * @return
     */
    @RequestMapping(value="/admin/machinery",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("machinery:add")
    public ResponseBean taMachineryAdd(@ApiParam("保存内容") @RequestBody TaMachinery taMachinery) throws Exception{
        SysUser sysUser = currentUser();

        if (StringUtils.isEmpty(taMachinery.getOrgId())) {
            return ResponseBean.error("请设置农机归属");
        }

        checkOrgAccess(taMachinery.getOrgId());

        taMachinery.setMachineryId(null);
        taMachinery.setCreateUser(sysUser.getUserId());

        if (iTaMachineryService.saveAll(taMachinery)){
            iTaOrgService.updateNum(taMachinery.getOrgId(), "machinery_num", 1);
            return ResponseBean.success(taMachinery);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("machinery:delete")
    public ResponseBean taMachineryDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaMachinery taMachinery = iTaMachineryService.getExistBy("machinery_id", id, false, true);
        if (null == taMachinery) {
            return ResponseBean.success("success");
        }

        checkOrgAccess(taMachinery.getOrgId());

        if(iTaMachineryService.removeLogicById(id)){
            iTaOrgService.updateNum(taMachinery.getOrgId(), "machinery_num", -1);
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taMachinery 实体对象
     * @return
     */
    @RequestMapping(value="/admin/machinery/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("machinery:update")
    public ResponseBean taMachineryUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaMachinery taMachinery) throws Exception{
        if (StringUtils.isEmpty(taMachinery.getOrgId())) {
            return ResponseBean.error("请设置农机归属");
        }

        checkOrgAccess(taMachinery.getOrgId());

        taMachinery.setMachineryId(id);

        if (iTaMachineryService.updateAll(taMachinery)){
            return ResponseBean.success(taMachinery);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMachineryGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaMachineryService.getDetail(id));
    }


    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/machinery-summary/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getSummaryDetail(@ApiParam("客户端") @PathVariable String client,
                                         @ApiParam("客户端ID") @PathVariable String clientId,
                                         @ApiParam("对象ID") @PathVariable String id,
                                         @ApiParam("当前位置") @RequestParam(value ="location") String location,
                                         @ApiParam("是否查询附件") @RequestParam(value ="attached", defaultValue = "false") Boolean attached) throws Exception{
        return ResponseBean.success(iTaMachineryService.getSummaryDetail(id, location, attached));
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}/duty",method= RequestMethod.GET)
    @ApiOperation(value="校验车辆预约期状态", notes = "校验车辆预约期状态", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMachineryDuty(@ApiParam("对象ID") @PathVariable String id,
                                         @ApiParam(value = "预约时间", example = "2022-03-30") @RequestParam(value ="date") String date) throws Exception{

        List<TaOrder> taOrderList = iTaOrderService.checkMachineryDuty(id, date);

        return ResponseBean.success(taOrderList);
    }


    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}/status/{status}",method= RequestMethod.PUT)
    @ApiOperation(value="更新车辆状态", notes = "更新车辆状态", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean getMachineryDuty(@ApiParam("对象ID") @PathVariable String id,
                                         @ApiParam("车辆状态") @PathVariable Integer status) throws Exception{

        TaMachinery taMachinery = iTaMachineryService.getById(id);
        if (taMachinery == null) {
            return ResponseBean.error("未找到车辆信息");
        }

        taMachinery.setStatus(status);
        iTaMachineryService.updateById(taMachinery);

        return ResponseBean.success(taMachinery);
    }

    /**
     * 车辆实时预览
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}/preview",method= RequestMethod.GET)
    @ApiOperation(value="车辆实时预览", notes = "车辆实时预览", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean machineryPreview(@ApiParam("对象ID") @PathVariable String id) throws Exception{

        TdDevice tdDevice = iTdDeviceService.getExistBy("machinery_id", id, false, true);
        if (tdDevice == null) {
            return ResponseBean.error("当前车辆未绑定设备");
        }

        DeviceSummary deviceSummary = JSONObject.parseObject(tdDevice.getApiConfig(), DeviceSummary.class);
        TdHatcDevice hatcDevice = iTdHatcDeviceService.getById(deviceSummary.getDeviceId());

//        if (hatcDevice == null || null == hatcDevice.getDeviceStatus() || hatcDevice.getDeviceStatus() != 1) {
//            return ResponseBean.error("车辆设备离线");
//        }

        if (hatcDevice == null) {
            return ResponseBean.error("未同步设备");
        }

        String url = String.format("%s/preview?terminalID=%s&productKey=%s&channelNo=%d&netType=%d",
                hatcApiServer,
                StringUtils.urlEncode(hatcDevice.getTerminalId()),
                StringUtils.urlEncode(hatcDevice.getProductKey()),
                deviceSummary.getChannel(),
                deviceSummary.getNetType()
                );

        String respStr = httpUtils.get(url);
        ResponseBean responseBean = JSONObject.parseObject(respStr, ResponseBean.class);

        Map<String, Object> data = new HashMap<>();
        data.put("url", responseBean.getData());
        data.put("devSerialNum", hatcDevice.getTerminalId());
        data.put("productKey", hatcDevice.getProductKey());
        data.put("channel", deviceSummary.getChannel());
        data.put("streamType", 1);  // 0 主码流, 1 子码流
        data.put("code", 0);
        data.put("protocol", 0);

        return ResponseBean.success(data);
    }
}
