package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaMessageTemplateService;
import com.yunzhi.nanyang.entity.TaMessageTemplate;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 消息模板 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-04-11
 */

@Api(tags = "消息模板")
@RestController
@RequestMapping("/")
public class TaMessageTemplateController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMessageTemplateController.class);

    @Autowired
    public ITaMessageTemplateService iTaMessageTemplateService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taMessageTemplate",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMessageTemplateList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

		    IPage<TaMessageTemplate> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaMessageTemplate> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaMessageTemplate> result = iTaMessageTemplateService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taMessageTemplate 实体对象
     * @return
     */
    @RequestMapping(value="/taMessageTemplate",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taMessageTemplateAdd(@ApiParam("保存内容") @RequestBody TaMessageTemplate taMessageTemplate) throws Exception{

        if (iTaMessageTemplateService.save(taMessageTemplate)){
            return ResponseBean.success(taMessageTemplate);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taMessageTemplate/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taMessageTemplateDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iTaMessageTemplateService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taMessageTemplate 实体对象
     * @return
     */
    @RequestMapping(value="/taMessageTemplate/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taMessageTemplateUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody TaMessageTemplate taMessageTemplate) throws Exception{

        if (iTaMessageTemplateService.updateById(taMessageTemplate)){
            return ResponseBean.success(iTaMessageTemplateService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taMessageTemplate/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMessageTemplateGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaMessageTemplateService.getById(id));
    }
}
