package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaExtendContent;
import com.yunzhi.nanyang.service.ITaExtendContentService;
import com.yunzhi.nanyang.service.ITaNewsTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaNewsService;
import com.yunzhi.nanyang.entity.TaNews;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
    * 资讯表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "资讯表")
@RestController
@RequestMapping("/")
public class TaNewsController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaNewsController.class);

    @Autowired
    public ITaNewsService iTaNewsService;

    @Autowired
    public ITaNewsTypeService iTaNewsTypeService;

    @Autowired
    public ITaExtendContentService iTaExtendContentService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/news",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taNewsList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("名称") @RequestParam(value ="title", required = false) String title,
                                   @ApiParam("类型") @RequestParam(value ="typeId", required = false) String typeId) throws Exception{
//        SysUser sysUser = currentUser();
//        boolean isAdmin = Constants.ADMIN_ID.equals(sysUser.getUserId());

        IPage<TaNews> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaNews> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(title), "title", "%" + title + "%");
        queryWrapper.eq(!StringUtils.isEmpty(typeId), "type_id", typeId);
//        queryWrapper.in(!isAdmin, "org_id", getAccessOrgs(sysUser.getUserId()));
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<TaNews> result = iTaNewsService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/news",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taNewsList(@ApiParam("客户端") @PathVariable String client,
                                   @ApiParam("客户端ID") @PathVariable String clientId,
                                   @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("名称") @RequestParam(value ="title", required = false) String title,
                                   @ApiParam("类型") @RequestParam(value ="typeId", required = false) String typeId) throws Exception{
//        SysUser sysUser = currentUser();
//        boolean isAdmin = Constants.ADMIN_ID.equals(sysUser.getUserId());

        IPage<TaNews> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaNews> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(title), "title", "%" + title + "%");
        queryWrapper.eq(!StringUtils.isEmpty(typeId), "type_id", typeId);
//        queryWrapper.in(!isAdmin, "org_id", getAccessOrgs(sysUser.getUserId()));
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");

        IPage<TaNews> result = iTaNewsService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }


    /**
     * 保存对象
     * @param taNews 实体对象
     * @return
     */
    @RequestMapping(value="/admin/news",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("news:add")
    public ResponseBean taNewsAdd(@ApiParam("保存内容") @RequestBody TaNews taNews) throws Exception{
        SysUser sysUser = currentUser();

        if (StringUtils.isEmpty(taNews.getTitle()) || StringUtils.isEmpty(taNews.getTypeId())) {
            return ResponseBean.error("标题或者分类不能为空");
        }

        taNews.setTypeName(iTaNewsTypeService.getById(taNews.getTypeId()).getName());

        if (iTaNewsService.save(taNews)){
            iTaExtendContentService.editContent(Constants.TARGET_NEWS, taNews.getNewsId(), taNews.getContentList());
            return ResponseBean.success(taNews);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/news/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("news:delete")
    public ResponseBean taNewsDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaNewsService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taNews 实体对象
     * @return
     */
    @RequestMapping(value="/admin/news/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("news:update")
    public ResponseBean taNewsUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaNews taNews) throws Exception{
        taNews.setNewsId(id);

        if (StringUtils.isEmpty(taNews.getTitle()) || StringUtils.isEmpty(taNews.getTypeId())) {
            return ResponseBean.error("标题或者分类不能为空");
        }

        taNews.setTypeName(iTaNewsTypeService.getById(taNews.getTypeId()).getName());

//        iTaExtendContentService.editContent(Constants.TARGET_NEWS, id, taNews.getContentList());

        if (iTaNewsService.updateById(taNews)){
            return ResponseBean.success(taNews);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/news/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taNewsGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaNews taNews = iTaNewsService.getExistBy("news_id", id, false, true);
        if (null == taNews) {
            return ResponseBean.error("内存不存在");
        }

        List<TaExtendContent> contentList = iTaExtendContentService.listByTarget(Constants.TARGET_NEWS, id);
        taNews.setContentList(contentList);

        return ResponseBean.success(taNews);
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}//news/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetail(@ApiParam("客户端") @PathVariable String client,
                                  @ApiParam("客户端ID") @PathVariable String clientId,
                                  @ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaNews taNews = iTaNewsService.getExistBy("news_id", id, false, true);
        if (null == taNews) {
            return ResponseBean.error("内存不存在");
        }

        List<TaExtendContent> contentList = iTaExtendContentService.listByTarget(Constants.TARGET_NEWS, id);
        taNews.setContentList(contentList);

        return ResponseBean.success(taNews);
    }
}
