package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.service.ITaNewsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaNewsTypeService;
import com.yunzhi.nanyang.entity.TaNewsType;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 资讯分类表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "资讯分类表")
@RestController
@RequestMapping("/")
public class TaNewsTypeController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaNewsTypeController.class);

    @Autowired
    public ITaNewsTypeService iTaNewsTypeService;

    @Autowired
    public ITaNewsService iTaNewsService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/news-type",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taNewsTypeList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TaNewsType> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaNewsType> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "name", "%" + name + "%");
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<TaNewsType> result = iTaNewsTypeService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taNewsType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/news-type",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("news:add")
    public ResponseBean taNewsTypeAdd(@ApiParam("保存内容") @RequestBody TaNewsType taNewsType) throws Exception{
        taNewsType.setTypeId(null);

        if (StringUtils.isEmpty(taNewsType.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        int cnt = iTaNewsTypeService.countBy("name", taNewsType.getName(), true);
        if (cnt > 0) {
            return ResponseBean.error("名称重复");
        }


        if (iTaNewsTypeService.save(taNewsType)){
            return ResponseBean.success(taNewsType);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/news-type/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("news:delete")
    public ResponseBean taNewsTypeDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaNewsType taNewsType = iTaNewsTypeService.getExistBy("type_id", id, false, true);
        if (null == taNewsType) {
            return ResponseBean.success("success");
        }

        int cnt = iTaNewsService.countBy("type_id", id, true);
        if (cnt > 0) {
            return ResponseBean.error("请先移除当前分类下的资讯信息");
        }

        if(iTaNewsTypeService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taNewsType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/news-type/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("news:update")
    public ResponseBean taNewsTypeUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaNewsType taNewsType) throws Exception{
        taNewsType.setTypeId(id);

        if (StringUtils.isEmpty(taNewsType.getName())) {
            return ResponseBean.error("名称不能为空");
        }
        TaNewsType origin = iTaNewsTypeService.getByButNot("name", taNewsType.getName(), "type_id", id, true);
        if (origin != null) {
            return ResponseBean.error("名称重复");
        }

        if (iTaNewsTypeService.updateById(taNewsType)){
            return ResponseBean.success(iTaNewsTypeService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/news-type/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taNewsTypeGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaNewsTypeService.getExistBy("type_id", id, false, true));
    }
}
