package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderV3Request;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 订单表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "订单表")
@RestController
@RequestMapping("/")
public class TaOrderController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrderController.class);

    @Autowired
    public ITaOrgService iTaOrgService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaPersonService iTaPersonService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ITaMachineryService iTaMachineryService;

    @Autowired
    public ITaPayService iTaPayService;

    @Autowired
    public ITaDispatchService iTaDispatchService;

    @Autowired
    public ITaWorkMessageService iTaWorkMessageService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/order",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("是否预警") @RequestParam(value ="isWarn", defaultValue = "false") Boolean isWarn,
                                @ApiParam("机构") @RequestParam(value ="orgId", required = false) String orgId,
                                @ApiParam("订单号") @RequestParam(value ="orderNo", required = false) String orderNo,
                                @ApiParam("农机") @RequestParam(value ="machineryId", required = false) String machineryId,
                                @ApiParam("类型") @RequestParam(value ="typeId", required = false) String typeId,
                                @ApiParam("下单人") @RequestParam(value ="personName", required = false) String personName,
                                @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone,
                                @ApiParam("付款状态") @RequestParam(value ="payStatus", required = false) Integer payStatus,
                                @ApiParam("调度状态") @RequestParam(value ="dispatchStatus", required = false) Integer dispatchStatus,
                                @ApiParam("作业状态") @RequestParam(value ="workStatus", required = false) Integer workStatus,
                                @ApiParam("评价状态") @RequestParam(value ="isEvaluated", required = false) Integer isEvaluated) throws Exception{
        SysUser current = currentUser();
        boolean isAdmin = Constants.ADMIN_ID.equals(current.getUserId());
        checkOrgAccess(orgId);

        IPage<TaOrder> pg = new Page<>(pageNum, pageSize);
        IPage<TaOrder> result = iTaOrderService.getAdminPage(pg,
                isAdmin ? null : current.getUserId(),
                orgId,
                orderNo,
                machineryId,
                typeId,
                personName,
                phone,
                payStatus,
                dispatchStatus,
                workStatus,
                isEvaluated,
                isWarn);

        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/order",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taOrderList(@ApiParam("客户端") @PathVariable String client,
                                    @ApiParam("客户端ID") @PathVariable String clientId,
                                    @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									@ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("我的") @RequestParam(value = "mine", defaultValue = "false") Boolean isMine,
                                    @ApiParam("作业状态") @RequestParam(value = "workStatus", required = false) Integer workStatus,
                                    @ApiParam("关键字") @RequestParam(value = "q", required = false) String q) throws Exception{
        TaPerson taPerson = currentPerson();

        SysUser sysUser = null;
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            String userId = taPerson.getUserId();
            if (StringUtils.isEmpty(userId)) {
                return ResponseBean.error("未绑定农机手信息");
            }

            if (!Constants.ADMIN_ID.equals(userId)) {
                sysUser = iSysUserService.getExistBy("user_id", userId, false, true);
                if (null == sysUser) {
                    return ResponseBean.error("未绑定农机手信息");
                }
            }
        }

        IPage<TaOrder> pg = new Page<>(pageNum, pageSize);

        IPage<TaOrder> result = iTaOrderService.getClientPage(pg, isMine ? taPerson : null, sysUser, workStatus, q);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taOrder 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/order",method= RequestMethod.POST)
    @ApiOperation(value="生成订单", notes = "生成订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taOrderAdd(@ApiParam("客户端") @PathVariable String client,
                                   @ApiParam("客户端ID") @PathVariable String clientId,
                                   @ApiParam("保存内容") @RequestBody TaOrder taOrder,
                                   HttpServletRequest request) throws Exception{
        TaPerson taPerson = currentPerson();

        String orderNo = iTaOrderService.getNextOrderNO();

        taOrder.setOrderId(null);
        taOrder.setOrderNo(orderNo);
        taOrder.setPersonId(taPerson.getPersonId());

//        TaMachinery taMachinery = iTaMachineryService.getById(taOrder.getMachineryId());
//        taOrder.setOrgId(taMachinery.getOrgId());

        if (StringUtils.isEmpty(taOrder.getOrgId())) {
            return ResponseBean.error("未找到接单机构");
        }

        TaOrg taOrg = iTaOrgService.getById(taOrder.getOrgId());
        if (null == taOrg) {
            return ResponseBean.error("未找到接单机构");
        }

        if (iTaOrderService.save(taOrder)){
            iTaWorkMessageService.appoints(taOrder, taPerson);
            return ResponseBean.success(taOrder);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/order/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="取消订单", notes = "取消订单", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taOrderDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{

        TaPerson taPerson = currentPerson();
        //
        TaOrder taOrder = iTaOrderService.getById(id);
        if (null == taOrder || taOrder.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到订单信息");
        }

        if (!taOrder.getPersonId().equals(taPerson.getPersonId())) {
            return ResponseBean.error("无权操作");
        }

        if (taOrder.getPayStatus() == Constants.PAY_PAID) {
            return ResponseBean.error("订单已支付");
        }

        if(iTaOrderService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/order/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetail(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(getOrder(id));
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/order/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taOrderGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(getOrder(id));
    }

    private TaOrder getOrder(String id) throws Exception {
        TaOrder taOrder = iTaOrderService.getExistBy("order_id", id, false, true);
        if (null == taOrder) {
            throw new Exception("订单不存在");
        }

        TaOrg taOrg = iTaOrgService.getById(taOrder.getOrgId());
        if (null == taOrg) {
            throw new Exception("订单机构不存在");
        }

        TaDispatch taDispatch = iTaDispatchService.getExistBy("order_id", taOrder.getOrderId(), false, true);
        if (taDispatch != null) {
            taOrder.setWorkId(taDispatch.getWorkerId());
            taOrder.setWorkName(taDispatch.getWorkerName());
        }

        taOrder.setOrgName(taOrg.getName());

        TaPerson taPerson = iTaPersonService.getById(taOrder.getPersonId());
        taOrder.setPersonName(taPerson.getNickName());
        taOrder.setPhone(taPerson.getPhone());
        return taOrder;
    }
}
