package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.service.ISysUserDataScopeService;
import com.yunzhi.nanyang.service.ISysUserService;
import com.yunzhi.nanyang.service.ITaMachineryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaOrgService;
import com.yunzhi.nanyang.entity.TaOrg;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
    * 机构管理 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "机构管理")
@RestController
@RequestMapping("/")
public class TaOrgController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrgController.class);

    @Autowired
    public ITaOrgService iTaOrgService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ITaMachineryService iTaMachineryService;

    @Autowired
    public ISysUserDataScopeService iSysUserDataScopeService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/org",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("org:view")
    public ResponseBean taOrgList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                  @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                  @ApiParam("名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TaOrg> pg = new Page<>(pageNum, pageSize);
        SysUser sysUser = currentUser();

        IPage<TaOrg> result = iTaOrgService.getPageBy(pg, sysUser, name);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taOrg 实体对象
     * @return
     */
    @RequestMapping(value="/admin/org",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("org:add")
    public ResponseBean taOrgAdd(@ApiParam("保存内容") @RequestBody TaOrg taOrg) throws Exception{

        SysUser sysUser = currentUser();

        taOrg.setOrgId(null);
        taOrg.setCreateUser(sysUser.getUserId());

        if (iTaOrgService.saveWithDataScope(taOrg)){
            return ResponseBean.success(taOrg);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/org/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("org:delete")
    public ResponseBean taOrgDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{

        int cnt = iSysUserService.countBy("org_id", id, true);
        if (cnt > 0) {
            return ResponseBean.error("请先清空机构人员信息");
        }

        cnt = iTaMachineryService.countBy("org_id", id, true);
        if (cnt > 0) {
            return ResponseBean.error("请先清空机构机构信息");
        }

        if(iTaOrgService.remvoeWithDataScope(id, currentUser().getUserId())){
            Map<String, Object> filter = new HashMap<>();
            filter.put("org_id", id);
            iSysUserDataScopeService.removeByMap(filter);

            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("无权限或者ID不正确", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taOrg 实体对象
     * @return
     */
    @RequestMapping(value="/admin/org/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("org:update")
    public ResponseBean taOrgUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaOrg taOrg) throws Exception{

        if (!iTaOrgService.canAccessData(id, currentUser().getUserId())) {
            return ResponseBean.error("无权限或者ID不正确");
        }

        if (taOrg.getStatus().equals(Constants.STATUS_DELETE)) {
            return ResponseBean.error("未找到数据");
        }

        // 不支持修改如下字段
        taOrg.setCreateUser(null);
        taOrg.setCreateDate(null);

        if (iTaOrgService.updateById(taOrg)){
            return ResponseBean.success(iTaOrgService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/org/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("org:view")
    public ResponseBean taOrgGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if (!iTaOrgService.canAccessData(id, currentUser().getUserId())) {
            return ResponseBean.error("无权限或者ID不正确");
        }

        return ResponseBean.success(iTaOrgService.getById(id));
    }
}
