package com.yunzhi.nanyang.controller;

import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderV3Result;
import com.github.binarywang.wxpay.bean.result.enums.TradeTypeEnum;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.service.WxPayService;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.service.ITaOrderService;
import com.yunzhi.nanyang.service.ITaPayService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@Api(tags = "支付")
@RestController
@RequestMapping("/")
public class TaPayController extends BaseController {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    ITaPayService iTaPayService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @RequestMapping(value="/{client}/{clientId}/order/{id}/prepay",method= RequestMethod.POST)
    @ApiOperation(value="预支付", notes = "预支付", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean prepay(@ApiParam("客户端") @PathVariable String client,
                               @ApiParam("客户端ID") @PathVariable String clientId,
                               @ApiParam("订单ID") @PathVariable String id,
                               @ApiParam("支付相关") @RequestBody TaOrder payOrder) throws Exception{
        if (null == payOrder || StringUtils.isEmpty(payOrder.getPayType())) {
            return ResponseBean.error("请选择支付方式");
        }

        TaOrder taOrder = iTaOrderService.getExistBy("order_id", id, false, true);
        if (null == taOrder) {
            return ResponseBean.error("未找到订单信息");
        }

        // 设置支付方式
        taOrder.setPayType(payOrder.getPayType());

        if (taOrder.getPayStatus() != Constants.PAY_READY) {
            return ResponseBean.error("订单已支付");
        }

        iTaOrderService.updateById(taOrder);

        // 是否微信小程序
        boolean isMiniapp = "wx".equals(client);
        // 是否微信支付
        boolean isWxpay = "wx".equals(payOrder.getPayType());

        // 如果是微信小程序支付
        if (isMiniapp) {
            WxPayUnifiedOrderV3Result payResult = iTaPayService.createWxPay(client, clientId, taOrder);
            WxPayConfig payConfig = wxUtils.getMiniPayService().getConfig();
            WxPayUnifiedOrderV3Result.JsapiResult result = payResult.getPayInfo(TradeTypeEnum.JSAPI, payConfig.getAppId(), payConfig.getMchId(), payConfig.getPrivateKey());
            return ResponseBean.success(result);
        } else if (isWxpay) {
            // 如果是 APP 微信支付
            WxPayUnifiedOrderV3Result payResult = iTaPayService.createWxPay(client, clientId, taOrder);
            WxPayConfig payConfig = wxUtils.getAndroidPayService().getConfig();
            WxPayUnifiedOrderV3Result.AppResult result = payResult.getPayInfo(TradeTypeEnum.APP, payConfig.getAppId(), payConfig.getMchId(), payConfig.getPrivateKey());
            return ResponseBean.success(result);
        } else {
            // 如果是 APP 支付宝支付
            AlipayTradeAppPayResponse aliPayResponse = iTaPayService.createAliPay(client, clientId, taOrder);
            return ResponseBean.success(aliPayResponse.getBody());
        }
    }


    @RequestMapping(value="/{client}/{clientId}/order/{id}/refund",method= RequestMethod.PUT)
    @ApiOperation(value="申请退款", notes = "申请退款", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean refund(@ApiParam("客户端") @PathVariable String client,
                               @ApiParam("客户端ID") @PathVariable String clientId,
                               @ApiParam("订单ID") @PathVariable String id) throws Exception{
        return ResponseBean.error("暂不支持退款操作");
//        TaOrder taOrder = iTaOrderService.getExistBy("order_id", id, false, true);
//        if (null == taOrder) {
//            return ResponseBean.error("未找到订单信息");
//        }
//
//        if (taOrder.getPayStatus() != Constants.PAY_PAID) {
//            return ResponseBean.error("订单未支付");
//        }
//        if (taOrder.getWorkStatus() != Constants.WORK_READY) {
//            return ResponseBean.error("订单已作业, 不能退款");
//        }
//
//        iTaPayService.refund(client, clientId, taOrder);
//        return ResponseBean.success("success");
    }
}
