package com.yunzhi.nanyang.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.service.ISysUserService;
import com.yunzhi.nanyang.vo.PhoneParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.nanyang.service.ITaPersonService;
import com.yunzhi.nanyang.entity.TaPerson;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>
    * 人员表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "人员表")
@RestController
@RequestMapping("/")
public class TaPersonController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaPersonController.class);

    @Autowired
    public ITaPersonService iTaPersonService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public SMSCaptcha smsCaptcha;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/person",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taPersonList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("身份") @RequestParam(value ="identity", required = false) String identity,
                                     @ApiParam("昵称") @RequestParam(value ="nickName", required = false) String nickName,
                                     @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone) throws Exception{

        IPage<TaPerson> pg = new Page<>(pageNum, pageSize);

        IPage<TaPerson> result = iTaPersonService.getPageBy(pg, identity, nickName, phone);
        return ResponseBean.success(result);
    }

//    /**
//     * 保存对象
//     * @param taPerson 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taPerson",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taPersonAdd(@ApiParam("保存内容") @RequestBody TaPerson taPerson) throws Exception{
//
//        if (iTaPersonService.save(taPerson)){
//            return ResponseBean.success(taPerson);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taPerson/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taPersonDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaPersonService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//

    /**
     * 更新人员信息
     * @param client  客户端
     * @param clientId  客户端ID
     * @param id  实体ID
     * @param taPerson 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/person/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新人员信息", notes = "更新人员信息", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taPersonUpdate(@ApiParam("客户端") @PathVariable String client,
                                       @ApiParam("客户端ID") @PathVariable String clientId,
                                       @ApiParam("对象ID") @PathVariable String id,
                                       @ApiParam("更新内容") @RequestBody TaPerson taPerson) throws Exception{

        TaPerson current = currentPerson();
        if (!id.equals(current.getPersonId())) {
            return ResponseBean.error("待更新人员ID不正确");
        }

        if (StringUtils.isEmpty(taPerson.getPhone())) {
            return ResponseBean.error("手机号码为空");
        }

        if (StringUtils.isEmpty(taPerson.getNickName())) {
            return ResponseBean.error("姓名为空");
        }

//        boolean isOk = smsCaptcha.validate(phoneParam.getPhone(), phoneParam.getCaptcha());
//        if (!isOk) {
//            return ResponseBean.error("验证码不正确或者已过期");
//        }

        taPerson.setPersonId(id);

        // 如果是农机手
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            // 关联系统人员
            SysUser sysUser = iSysUserService.getExistBy("phone", taPerson.getPhone(), false, true);
            if (sysUser != null && StringUtils.isEmpty(taPerson.getUserId())) {
                taPerson.setUserId(sysUser.getUserId());
            }
        }

        if (iTaPersonService.updateById(taPerson)){
            return ResponseBean.success(taPerson);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param clientId  客户端ID
     * @param id  实体ID
     * @param phoneParam 实体对象
     * @return
     */
    @RequestMapping(value="/wx/{clientId}/person/{id}/phone",method= RequestMethod.PUT)
    @ApiOperation(value="更新电话", notes = "更新电话", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean updatePhone(@ApiParam("客户端ID") @PathVariable String clientId,
                                    @ApiParam("对象ID") @PathVariable String id,
                                    @ApiParam("更新内容") @RequestBody PhoneParam phoneParam) throws Exception{

        TaPerson taPerson = currentPerson();
        if (!id.equals(taPerson.getPersonId())) {
            return ResponseBean.error("待更新人员ID不正确");
        }

        if (StringUtils.isEmpty(phoneParam.getPhone())) {
            return ResponseBean.error("手机号码为空");
        }

        boolean isOk = smsCaptcha.validate(phoneParam.getPhone(), phoneParam.getCaptcha());
        if (!isOk) {
            return ResponseBean.error("验证码不正确或者已过期");
        }

        if (!StringUtils.isEmpty(taPerson.getPhone())) {
            return ResponseBean.error("暂不支持手机号修改");
        }

        // 先去查询是否已经注册过
        TaPerson origin = iTaPersonService.getByPhone(clientId, phoneParam.getPhone());
        if (origin != null && !origin.getPersonId().equals(id)) {
            // 找到同手机号人员, 说明是 app 端注册过的
            // 那么把小程序端人员信息移植到 app 端人员上去
            origin.setAppId(taPerson.getAppId());
            origin.setOpenid(taPerson.getOpenid());

            if (StringUtils.isEmpty(origin.getNickName())) {
                origin.setNickName(taPerson.getNickName());
            }
            if (StringUtils.isEmpty(origin.getAvatar())) {
                origin.setAvatar(taPerson.getAvatar());
            }
            if (null != origin.getSex()) {
                origin.setSex(taPerson.getSex());
            }

            // 先把小程序端人员删除
            taPerson.setOpenid("deleted");
            taPerson.setStatus(Constants.STATUS_DELETE);
            iTaPersonService.updateById(taPerson);
            // 再赋值 taPerson , 后续再处理 taPerson 实际上就是处理的 app 人员了
            taPerson = origin;
        }

        taPerson.setPhone(phoneParam.getPhone());

        // 如果是农机手
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            // 关联系统人员
            SysUser sysUser = iSysUserService.getExistBy("phone", phoneParam.getPhone(), false, true);
            if (sysUser != null && StringUtils.isEmpty(taPerson.getUserId())) {
                taPerson.setUserId(sysUser.getUserId());
            }
        }

        if (iTaPersonService.updateById(taPerson)){
            StpUtil.logout();
            StpUtil.login(taPerson.getPersonId(), clientId);
            Map<String, Object> res = new HashMap<>();
            res.put("person", taPerson);
            res.put("token", StpUtil.getTokenValue());
            return ResponseBean.success(res);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     */
    @RequestMapping(value="/{client}/{clientId}/person/current",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taPersonGet() throws Exception{
        return ResponseBean.success(currentPerson());
    }
}
