package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.service.ITaMachineryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaRegionService;
import com.yunzhi.nanyang.entity.TaRegion;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 归属区域表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "归属区域表")
@RestController
@RequestMapping("/")
public class TaRegionController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRegionController.class);

    @Autowired
    public ITaRegionService iTaRegionService;

    @Autowired
    public ITaMachineryService iTaMachineryService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/region",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("region:view")
    public ResponseBean taRegionList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("名称") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TaRegion> pg = new Page<>(pageNum, pageSize);

        IPage<TaRegion> result = iTaRegionService.getPageBy(pg, currentUser().getUserId(), name);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taRegion 实体对象
     * @return
     */
    @RequestMapping(value="/admin/region",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("region:add")
    public ResponseBean taRegionAdd(@ApiParam("保存内容") @RequestBody TaRegion taRegion) throws Exception{

        taRegion.setRegionId(null);

        if (StringUtils.isEmpty(taRegion.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        TaRegion origin = iTaRegionService.getExistBy("name", taRegion.getName(), false, true);
        if (null != origin) {
            return ResponseBean.error("名称重复");
        }

        if (iTaRegionService.save(taRegion)){
            return ResponseBean.success(taRegion);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/region/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("region:delete")
    public ResponseBean taRegionDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaRegionService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taRegion 实体对象
     * @return
     */
    @RequestMapping(value="/admin/region/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("region:update")
    public ResponseBean taRegionUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaRegion taRegion) throws Exception {

        taRegion.setRegionId(id);

        if (StringUtils.isEmpty(taRegion.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        TaRegion other = iTaRegionService.getByButNot("name", taRegion.getName(), "region_id", id, true);
        if (other != null) {
            return ResponseBean.error("名称重复");
        }

        if (iTaRegionService.updateById(taRegion)){
            iTaMachineryService.updateRegionName(taRegion.getRegionId(), taRegion.getName());
            return ResponseBean.success(taRegion);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/region/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("region:view")
    public ResponseBean taRegionGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaRegionService.getExistBy("region_id", id, false, true));
    }
}
