package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaMachinery;
import com.yunzhi.nanyang.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaUserMachineryService;
import com.yunzhi.nanyang.entity.TaUserMachinery;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
    * 农机绑定 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "农机绑定")
@RestController
@RequestMapping("/")
public class TaUserMachineryController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaUserMachineryController.class);

    @Autowired
    public ITaUserMachineryService iTaUserMachineryService;

    @Autowired
    public ISysUserService iSysUserService;


    /**
     * 分页查询列表
     * @param userId
     * @return
     */
    @RequestMapping(value="/admin/user-machinery",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taUserMachineryList(@ApiParam("用户ID") @RequestParam(value ="userId") String userId) throws Exception{

        SysUser sysUser = iSysUserService.getExistBy("user_id", userId, false, true);
        if (null == sysUser) {
            return ResponseBean.error("未找到指定人员");
        }

        checkOrgAccess(sysUser.getOrgId());

        List<TaUserMachinery> result = iTaUserMachineryService.getByUser(userId);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param userMachineryList 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user/{userId}/machinery",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("user:update")
    public ResponseBean taUserMachineryAdd(@ApiParam("用户ID") @PathVariable String userId,
                                           @ApiParam("保存内容") @RequestBody List<TaUserMachinery> userMachineryList) throws Exception{

        SysUser sysUser = iSysUserService.getExistBy("user_id", userId, false, true);
        if (null == sysUser) {
            return ResponseBean.error("未找到指定人员");
        }

        checkOrgAccess(sysUser.getOrgId());

        if (iTaUserMachineryService.editWithUser(userId, userMachineryList)){
            return ResponseBean.success(userMachineryList);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

}
