package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 提现表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "提现表")
@RestController
@RequestMapping("/")
public class TaWithdrawalController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaWithdrawalController.class);

    @Autowired
    public ITaWithdrawalService iTaWithdrawalService;

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ITaBankCardService iTaBankCardService;

    @Autowired
    public ITaAccountService iTaAccountService;

    @Autowired
    public ITaOrgService iTaOrgService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/withdrawal",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWithdrawalList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @ApiParam("机构ID") @RequestParam(value ="orgId", required = false) String orgId,
                                         @ApiParam(value = "开始日期", example = "2022-03-01") @RequestParam(value ="start",required = false) String start,
                                         @ApiParam(value = "结束日期", example = "2022-03-31") @RequestParam(value ="end",required = false) String end,
                                         @ApiParam("申请人") @RequestParam(value ="userName", required = false) String userName,
                                         @ApiParam("审批状态") @RequestParam(value ="auditStatus", required = false) Integer auditStatus) throws Exception{

        checkOrgAccess(orgId);

        LocalDateTime startDate = DateUtils.getDayStart(start);
        LocalDateTime endDate = DateUtils.getDayEnd(end);

        IPage<TaWithdrawal> pg = new Page<>(pageNum, pageSize);
        IPage<TaWithdrawal> result = iTaWithdrawalService.getPageBy(pg, orgId, startDate, endDate, userName, auditStatus);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taWithdrawal 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/withdrawal",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taWithdrawalAdd(@ApiParam("保存内容") @RequestBody TaWithdrawal taWithdrawal) throws Exception{
        TaPerson taPerson = currentPerson();
        SysUser sysUser = checkWorker(taPerson);
        if (null == sysUser) {
            return ResponseBean.error("农机手信息异常");
        }

        if (null == sysUser.getIsOrgManager() || !sysUser.getIsOrgManager()) {
            return ResponseBean.error("暂无权限进行当前操作");
        }

        if (!StringUtils.isEmpty(taWithdrawal.getAccountCardNo())) {
            TaBankCard bankCard = iTaBankCardService.getById(taWithdrawal.getAccountCardNo());
            if (null == bankCard || !taPerson.getPersonId().equals(bankCard.getPersonId())) {
                return ResponseBean.error("银行卡校验异常");
            }
        }

        if (!iTaAccountService.checkAmount(sysUser.getOrgId(), taWithdrawal.getMoney(), true)) {
            return ResponseBean.error("账户余额不足");
        }

        taWithdrawal.setAuditStatus(Constants.AUDIT_READY);
        taWithdrawal.setUserId(sysUser.getUserId());
        taWithdrawal.setUserName(sysUser.getUserName());
        taWithdrawal.setOrgId(sysUser.getOrgId());
        taWithdrawal.setStatus(Constants.STATUS_NORMAL);

        if (iTaWithdrawalService.save(taWithdrawal)){
            return ResponseBean.success(taWithdrawal);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/worker/withdrawal/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taWithdrawalDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaPerson taPerson = currentPerson();
        SysUser sysUser = checkWorker(taPerson);
        if (null == sysUser) {
            return ResponseBean.error("农机手信息异常");
        }

        if (null == sysUser.getIsOrgManager() || !sysUser.getIsOrgManager()) {
            return ResponseBean.error("暂无权限进行当前操作");
        }

        TaWithdrawal taWithdrawal = iTaWithdrawalService.getExistBy("withdrawal_id", id, false, true);
        if (null == taWithdrawal || !sysUser.getUserId().equals(taWithdrawal.getUserId())) {
            return ResponseBean.error("暂无权限进行当前操作");
        }

        Integer auditStatus = taWithdrawal.getAuditStatus();
        if (null != auditStatus && auditStatus != Constants.AUDIT_READY) {
            return ResponseBean.error("申请已进入审批流程");
        }

        if(iTaWithdrawalService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taWithdrawal 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/withdrawal/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taWithdrawalUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaWithdrawal taWithdrawal) throws Exception{
        TaPerson taPerson = currentPerson();
        SysUser sysUser = checkWorker(taPerson);
        if (null == sysUser) {
            return ResponseBean.error("农机手信息异常");
        }

        if (null == sysUser.getIsOrgManager() || !sysUser.getIsOrgManager()) {
            return ResponseBean.error("暂无权限进行当前操作");
        }

        TaWithdrawal origin = iTaWithdrawalService.getExistBy("withdrawal_id", id, false, true);
        if (null == origin || !sysUser.getUserId().equals(origin.getUserId())) {
            return ResponseBean.error("暂无权限进行当前操作");
        }

        Integer auditStatus = origin.getAuditStatus();
        if (null != auditStatus && auditStatus != Constants.AUDIT_READY) {
            return ResponseBean.error("申请已进入审批流程");
        }

        // 只允许修改 金额与卡号
        if (!StringUtils.isEmpty(taWithdrawal.getAccountCardNo())) {
            TaBankCard bankCard = iTaBankCardService.getById(taWithdrawal.getAccountCardNo());
            if (null == bankCard || !taPerson.getPersonId().equals(bankCard.getPersonId())) {
                return ResponseBean.error("银行卡校验异常");
            }
        }

        if (!iTaAccountService.checkAmount(origin.getOrgId(), taWithdrawal.getMoney(), true)) {
            return ResponseBean.error("账户余额不足");
        }

        origin.setMoney(taWithdrawal.getMoney());
        origin.setAccountCardNo(taWithdrawal.getAccountCardNo());
        origin.setAccountCardType(taWithdrawal.getAccountCardType());

        if (iTaWithdrawalService.updateById(origin)){
            return ResponseBean.success(iTaWithdrawalService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


    /**
     * 修改对象
     * @param id  实体ID
     * @param taWithdrawal 实体对象
     * @return
     */
    @RequestMapping(value="/admin/withdrawal/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="审批", notes = "审批", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taWithdrawalAudit(@ApiParam("对象ID") @PathVariable String id,
                                          @ApiParam("更新内容") @RequestBody TaWithdrawal taWithdrawal) throws Exception{
        SysUser sysUser = currentUser();

        TaWithdrawal origin = iTaWithdrawalService.getExistBy("withdrawal_id", id, false, true);
        if (null == origin) {
            return ResponseBean.error("申请不存在");
        }

        String orgId = origin.getOrgId();
        checkOrgAccess(orgId);

        // 不能重复审批, 因为审批有账户操作
        Integer auditStatus = origin.getAuditStatus();
        if (null != auditStatus && auditStatus > Constants.AUDIT_READY) {
            return ResponseBean.error("审批流程已走完");
        }

        if (!iTaAccountService.checkAmount(orgId, taWithdrawal.getMoney(), false) && Constants.AUDIT_AGREE == taWithdrawal.getAuditStatus()) {
            return ResponseBean.error("账户余额不足");
        }

        // 只允许修改审批相关信息
        origin.setAuditUser(sysUser.getUserId());
        origin.setAuditUserName(sysUser.getUserName());
        origin.setAuditStatus(taWithdrawal.getAuditStatus());
        origin.setAuditRemark(taWithdrawal.getAuditRemark());
        origin.setAuditDate(LocalDateTime.now());

        if (iTaWithdrawalService.audit(origin)){
            return ResponseBean.success(iTaWithdrawalService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/withdrawal/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWithdrawalGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaWithdrawal taWithdrawal = iTaWithdrawalService.getExistBy("withdrawal_id", id, false, true);
        if (null == taWithdrawal) {
            return ResponseBean.error("提现申请不存在");
        }

        SysUser user = iSysUserService.getById(taWithdrawal.getUserId());
        taWithdrawal.setPhone(user.getPhone());

        String orgId = taWithdrawal.getOrgId();
        checkOrgAccess(orgId);

        TaOrg taOrg = iTaOrgService.getById(orgId);
        taWithdrawal.setOrgName(taOrg.getName());

        TaBankCard taBankCard = iTaBankCardService.getById(taWithdrawal.getAccountCardNo());
        taWithdrawal.setBankCard(taBankCard);

        TaAccount taAccount = iTaAccountService.getUserAccount(taWithdrawal.getOrgId());
        taWithdrawal.setAmountLeft(taAccount.getAmounts());

        return ResponseBean.success(taWithdrawal);
    }
}
