package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 作业表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "作业表")
@RestController
@RequestMapping("/")
public class TaWorkJobController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaWorkJobController.class);

    @Autowired
    public ITaWorkJobService iTaWorkJobService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaDispatchService iTaDispatchService;

    @Autowired
    public ITaOrgService iTaOrgService;

    @Autowired
    public ITaMachineryService iTaMachineryService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/work-job",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam(value = "合作社ID") @RequestParam(value ="orgId", required = false) String orgId,
                                @ApiParam(value = "起始时间", example = "2022-03-01") @RequestParam(value ="start", required = false) String start,
                                @ApiParam(value = "结束时间", example = "2022-03-31") @RequestParam(value ="end", required = false) String end,
                                @ApiParam(value = "订单号") @RequestParam(value ="orderNo", required = false) String orderNo,
                                @ApiParam(value = "农机型号ID") @RequestParam(value ="machineryTypeId", required = false) String machineryTypeId,
                                @ApiParam(value = "农机ID") @RequestParam(value ="machineryId", required = false) String machineryId,
                                @ApiParam(value = "农机手名称") @RequestParam(value ="workerName", required = false) String workerName) throws Exception{
        checkOrgAccess(orgId);

        LocalDateTime startDate = DateUtils.getDayStart(start);
        LocalDateTime endDate = DateUtils.getDayEnd(end);

        Integer status = Constants.WORK_DONE;

        IPage<TaWorkJob> pg = new Page<>(pageNum, pageSize);
        IPage<TaWorkJob> result = iTaWorkJobService.getAdminPageBy(pg, orgId, startDate, endDate, status, orderNo, machineryTypeId, machineryId, workerName);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/work-job",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWorkJobList(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("是否我的") @RequestParam(value ="mine", defaultValue = "true") Boolean isMine) throws Exception{
        TaPerson taPerson = currentPerson();
        if (isMine && StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        String userId = isMine ? taPerson.getUserId() : null;

        IPage<TaWorkJob> pg = new Page<>(pageNum, pageSize);
        IPage<TaWorkJob> result = iTaWorkJobService.getPageBy(pg, userId);
        return ResponseBean.success(result);
    }

    /**
     * 暂停/开启
     * @param id  实体ID
     * @param mode 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/work-job/{id}/pause",method= RequestMethod.PUT)
    @ApiOperation(value="暂停/开启", notes = "暂停/开启", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taWorkJobUpdate(@ApiParam("客户端") @PathVariable String client,
                                        @ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("是否暂停") @RequestParam("mode") Boolean mode) throws Exception{

        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob taWorkJob = iTaWorkJobService.getExistBy("job_id", id, false, true);
        if (!taPerson.getUserId().equals(taWorkJob.getWorkerId())) {
            return ResponseBean.error("无权操作");
        }

        if (Constants.WORK_PAUSE < taWorkJob.getStatus() || Constants.WORK_READY > taWorkJob.getStatus()) {
            return ResponseBean.error("非工作状态, 不能操作");
        }

        TaOrder taOrder = iTaOrderService.getExistBy("order_id", taWorkJob.getOrderId(), false, true);
        if (taOrder == null) {
            return ResponseBean.error("校验订单失败");
        }

//        if (Constants.PAY_PAID != taOrder.getPayStatus()) {
//            return ResponseBean.error("订单未支付或者已退款");
//        }

//        int status = mode ? Constants.WORK_DOING : Constants.WORK_PAUSE;
        int status = Constants.WORK_DOING;

        taWorkJob.setStatus(status);
        taWorkJob.setStartDate(LocalDateTime.now());

        // 联动状态
        iTaOrderService.updateStatus(taWorkJob.getOrderId(), "work_status", Constants.WORK_DOING);
        iTaMachineryService.updateStatus(taWorkJob.getMachineryId(), "job_status", Constants.WORK_DOING);

        if (iTaWorkJobService.updateById(taWorkJob)){
            return ResponseBean.success(taWorkJob);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 暂停/开启
     * @param id  实体ID
     * @param taWorkJob 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/work-job/{id}/stop",method= RequestMethod.PUT)
    @ApiOperation(value="完成工作", notes = "完成工作", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean stopJob(@ApiParam("客户端") @PathVariable String client,
//                              @ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("对象ID") @PathVariable String id,
                                @ApiParam("保存内容") @RequestBody TaWorkJob taWorkJob) throws Exception{

        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob origin = iTaWorkJobService.getExistBy("job_id", id, false, true);
        if (origin == null || !taPerson.getUserId().equals(taWorkJob.getWorkerId())) {
            return ResponseBean.error("无权操作");
        }

        if (Constants.WORK_PAUSE != taWorkJob.getStatus() && Constants.WORK_DOING != taWorkJob.getStatus()) {
            return ResponseBean.error("非工作状态, 不能操作");
        }

        origin.setStatus(Constants.WORK_DONE);
        origin.setArea(taWorkJob.getArea());
        origin.setEndDate(LocalDateTime.now());

        // 更新订单
        TaOrder taOrder = iTaOrderService.getExistBy("order_id", origin.getOrderId(), false, true);
        if (null == taOrder) {
            return ResponseBean.error("校验订单失败");
        }

        int area = StringUtils.isEmpty(taWorkJob.getArea()) ? 0 : Integer.valueOf(taWorkJob.getArea());
        int charges = taOrder.getPrice() * area;
        taOrder.setAmount(area);
        taOrder.setCharges(charges);
        taOrder.setWorkStatus(Constants.WORK_DONE);
        iTaOrderService.updateById(taOrder);
        iTaOrgService.updateNum(taOrder.getOrgId(), "order_num", 1);

//        iTaOrderService.updateStatus(taWorkJob.getOrderId(), "work_status", Constants.WORK_DONE);
        iTaMachineryService.updateStatus(taWorkJob.getMachineryId(), "job_status", Constants.WORK_DONE);

        if (iTaWorkJobService.updateById(origin)){
            return ResponseBean.success(origin);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/work-job/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWorkJobGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob taWorkJob = iTaWorkJobService.getDetailBy(id);

        return ResponseBean.success(taWorkJob);
    }
}
