package com.yunzhi.nanyang.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.TaMachinery;
import com.yunzhi.nanyang.entity.TdHatcDevice;
import com.yunzhi.nanyang.service.ITaMachineryService;
import com.yunzhi.nanyang.service.ITdHatcDeviceService;
import com.yunzhi.nanyang.vo.DeviceSummary;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITdDeviceService;
import com.yunzhi.nanyang.entity.TdDevice;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 设备表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "设备表")
@RestController
@RequestMapping("/")
public class TdDeviceController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdDeviceController.class);

    @Autowired
    public ITdDeviceService iTdDeviceService;

    @Autowired
    public ITdHatcDeviceService iTdHatcDeviceService;

    @Autowired
    public ITaMachineryService iTaMachineryService;

//
//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/tdDevice",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean tdDeviceList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//		    IPage<TdDevice> pg = new Page<>(pageNum, pageSize);
//            QueryWrapper<TdDevice> queryWrapper = new QueryWrapper<>();
//            queryWrapper.orderByDesc("create_date");
//
//            IPage<TdDevice> result = iTdDeviceService.page(pg, queryWrapper);
//            return ResponseBean.success(result);
//    }

    /**
     * 保存对象
     * @param tdDevice 实体对象
     * @return
     */
    @RequestMapping(value="/admin/device",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean tdDeviceAdd(@ApiParam("保存内容") @RequestBody TdDevice tdDevice) throws Exception{

        boolean isNew = StringUtils.isEmpty(tdDevice.getDeviceId());

        if (StringUtils.isEmpty(tdDevice.getMachineryId())) {
            return ResponseBean.error("未绑定农机");
        }

        TdDevice origin = iTdDeviceService.getExistBy("machinery_id", tdDevice.getMachineryId(), false, true);
        if (isNew) {
            // 如果是新增操作
            if (null != origin) {
                return ResponseBean.error("该农机已绑定设备");
            }
        } else {
            if (null != origin) {
                // 如果非新增操作, 并且当前的 id 与 数据库 id 不一致
                if (!tdDevice.getDeviceId().equals(origin.getDeviceId())) {
                    return ResponseBean.error("该农机已绑定设备");
                }
            } else {
                isNew = true;
            }
        }

        if (StringUtils.isEmpty(tdDevice.getApiConfig())) {
            return ResponseBean.error("未绑定海康设备");
        }

        TaMachinery taMachinery = iTaMachineryService.getExistBy("machinery_id", tdDevice.getMachineryId(), false, true);
        if (taMachinery == null) {
            return ResponseBean.error("绑定农机不存在");
        }

        try {
            DeviceSummary deviceSummary = JSONObject.parseObject(tdDevice.getApiConfig(), DeviceSummary.class);
            if (null == deviceSummary) {
                return ResponseBean.error("未绑定海康设备");
            }

            if (StringUtils.isEmpty(deviceSummary.getDeviceId())) {
                return ResponseBean.error("未绑定海康设备");
            }

            TdHatcDevice hatcDevice = iTdHatcDeviceService.getById(deviceSummary.getDeviceId());
            if (null == hatcDevice) {
                return ResponseBean.error("绑定的海康设备不存在");
            }

            if (null == deviceSummary.getChannel() || deviceSummary.getChannel() < 1) {
                return ResponseBean.error("未设置通道号, 或者通道号不正确");
            }

            if (null == deviceSummary.getNetType() || deviceSummary.getNetType() < 0 || deviceSummary.getNetType() > 3) {
                return ResponseBean.error("未设置网络类型, 或者网络类型不正确");
            }

            tdDevice.setDeviceNo(hatcDevice.getTerminalId());
            tdDevice.setDeviceType(hatcDevice.getProductKey());
            taMachinery.setOnlineStatus(hatcDevice.getDeviceStatus());
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error("海康设备参数无效");
        }

        boolean isOk = isNew ? iTdDeviceService.save(tdDevice) : iTdDeviceService.updateById(tdDevice);
        iTaMachineryService.updateById(taMachinery);

        if (isOk){
            return ResponseBean.success(tdDevice);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/tdDevice/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean tdDeviceDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTdDeviceService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param tdDevice 实体对象
//     * @return
//     */
//    @RequestMapping(value="/tdDevice/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean tdDeviceUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TdDevice tdDevice) throws Exception{
//
//        if (iTdDeviceService.updateById(tdDevice)){
//            return ResponseBean.success(iTdDeviceService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery/{id}/device",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdDeviceGet(@ApiParam("农机ID") @PathVariable String id) throws Exception{
        TdDevice tdDevice = iTdDeviceService.getExistBy("machinery_id", id, false, true);
        return ResponseBean.success(tdDevice);
    }
}
