package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITdHatcDeviceService;
import com.yunzhi.nanyang.entity.TdHatcDevice;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 海康设备表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-05-11
 */

@Api(tags = "海康设备表")
@RestController
@RequestMapping("/")
public class TdHatcDeviceController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdHatcDeviceController.class);

    @Autowired
    public ITdHatcDeviceService iTdHatcDeviceService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/hatc-device",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("终端ID") @RequestParam(value ="terminalId", required = false) String terminalId) throws Exception{

        IPage<TdHatcDevice> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TdHatcDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(terminalId), "terminal_id", StringUtils.like(terminalId));
        queryWrapper.orderByDesc("terminal_id");

        IPage<TdHatcDevice> result = iTdHatcDeviceService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/hatc-device/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdHatcDeviceGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTdHatcDeviceService.getById(id));
    }
}
