package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.service.ITaMachineryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITdMachineryTypeService;
import com.yunzhi.nanyang.entity.TdMachineryType;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 农机类型表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "农机类型表")
@RestController
@RequestMapping("/")
public class TdMachineryTypeController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdMachineryTypeController.class);

    @Autowired
    public ITdMachineryTypeService iTdMachineryTypeService;

    @Autowired
    public ITaMachineryService iTaMachineryService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/machinery-type",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdMachineryTypeList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                            @ApiParam("单页数据量") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TdMachineryType> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TdMachineryType> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "name", "%" + name + "%");
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<TdMachineryType> result = iTdMachineryTypeService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }


    /**
     * 移动端列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/machinery-type",method= RequestMethod.GET)
    @ApiOperation(value="移动端列表", notes = "移动端列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("客户端") @PathVariable String client,
                                @ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("单页数据量") @RequestParam(value ="name", required = false) String name) throws Exception{

        IPage<TdMachineryType> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TdMachineryType> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "name", "%" + name + "%");
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        IPage<TdMachineryType> result = iTdMachineryTypeService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param tdMachineryType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/machinery-type",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("machinetype:add")
    public ResponseBean tdMachineryTypeAdd(@ApiParam("保存内容") @RequestBody TdMachineryType tdMachineryType) throws Exception{

        tdMachineryType.setTypeId(null);

        if (StringUtils.isEmpty(tdMachineryType.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        int cnt = iTdMachineryTypeService.countBy("name", tdMachineryType.getName(), true);
        if (cnt > 0) {
            return ResponseBean.error("名称重复");
        }

        if (iTdMachineryTypeService.save(tdMachineryType)){
            return ResponseBean.success(tdMachineryType);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery-type/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("machinetype:delete")
    public ResponseBean tdMachineryTypeDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{

        TdMachineryType tdMachineryType = iTdMachineryTypeService.getExistBy("type_id", id, false, true);
        if (null == tdMachineryType) {
            return ResponseBean.success("success");
        }

        int cnt = iTaMachineryService.countBy("type_id", id, true);
        if (cnt > 0) {
            return ResponseBean.error("请先移除当前分类下的农机信息");
        }


        if(iTdMachineryTypeService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param tdMachineryType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/machinery-type/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("machinetype:update")
    public ResponseBean tdMachineryTypeUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TdMachineryType tdMachineryType) throws Exception{
        tdMachineryType.setTypeId(id);

        if (StringUtils.isEmpty(tdMachineryType.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        TdMachineryType origin = iTdMachineryTypeService.getByButNot("name", tdMachineryType.getName(), "type_id", id, true);
        if (origin != null) {
            return ResponseBean.error("名称重复");
        }

        if (iTdMachineryTypeService.updateById(tdMachineryType)){
            iTaMachineryService.updateTypeName(tdMachineryType.getTypeId(), tdMachineryType.getName());
            return ResponseBean.success(tdMachineryType);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/machinery-type/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdMachineryTypeGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTdMachineryTypeService.getExistBy("type_id", id, false, true));
    }
}
