package com.yunzhi.nanyang.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import cn.dev33.satoken.stp.StpUtil;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysMiniapp;
import com.yunzhi.nanyang.entity.TaMessageTemplate;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.service.ISysMiniappService;
import com.yunzhi.nanyang.service.ITaMessageTemplateService;
import com.yunzhi.nanyang.service.ITaPersonService;
import com.yunzhi.nanyang.vo.LoginParam;
import com.yunzhi.nanyang.vo.WxMaAuthParam;
import com.yunzhi.nanyang.vo.WxMaPreload;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(tags = "小程序登入/登出")
@RestController
@RequestMapping("/wx/{clientId}")
public class WxMaController extends BaseController {

    @Autowired
    ISysMiniappService iSysMiniappService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaMessageTemplateService iTaMessageTemplateService;

    @Autowired
    SMSCaptcha smsCaptcha;

    @Autowired
    WxUtils wxUtils;

    private boolean checkLoginParam(LoginParam loginParam) {
        return !StringUtils.isEmpty(loginParam.getCode())
                && !StringUtils.isEmpty(loginParam.getUserName())
                && !StringUtils.isEmpty(loginParam.getPassword());
    }

    private WxMaService getService(String clientId) throws Exception {
        SysMiniapp miniapp = iSysMiniappService.getByClient(clientId);
        if (null == miniapp || miniapp.getStatus() != Constants.STATUS_NORMAL) {
            throw new Exception("未找到有效的小程序配置");
        }

        return wxUtils.getService(miniapp.getClient());
    }

    @GetMapping("/preload")
    @ApiOperation(value="小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean preload(@ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("小程序预加载参数") @RequestParam(required = false) WxMaPreload wxMaPreload) throws Exception {

        if (Constants.CLIENT_WORKER.equals(clientId)) {
            List<TaMessageTemplate> list = iTaMessageTemplateService.getByTplType(Constants.MESSAGE_MINIAPP);
            return ResponseBean.success(new HashMap<String, Object>(){{
                put("tpls", list);
            }});
        }

        return ResponseBean.success(null);
    }

    /**
     * 小程序
     * @param loginParam
     * @return
     * @throws Exception
     */
    @PostMapping("/login")
    @ApiOperation(value="小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean appLogin(@ApiParam("客户端ID") @PathVariable String clientId,
                                 @ApiParam("登录参数") @RequestBody LoginParam loginParam) throws Exception {

        if (StringUtils.isEmpty(loginParam.getCode())) {
            return ResponseBean.error("CODE 不能为空");
        }

        SysMiniapp miniapp = iSysMiniappService.getByClient(clientId);
        WxMaService service = getService(clientId);

        WxMaJscode2SessionResult sessionInfo = service.getUserService().getSessionInfo(loginParam.getCode());
        String openid = sessionInfo.getOpenid();
        String sessionKey = sessionInfo.getSessionKey();

        TaPerson taPerson = iTaPersonService.getByOpenId(miniapp.getAppId(), openid);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setAppId(miniapp.getAppId());
            taPerson.setOpenid(openid);
            taPerson.setIdentity(clientId);
            taPerson.setStatus(Constants.STATUS_NORMAL);
            taPerson.setCreateDate(LocalDateTime.now());
            iTaPersonService.save(taPerson);
        }

        // 如果是农机手
        if (Constants.CLIENT_WORKER.equals(clientId)) {
            iTaPersonService.tryBindUser(taPerson);
        }

        StpUtil.login(taPerson.getPersonId(), clientId);
        Map<String, Object> res = new HashMap<>();
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());
        res.put("sessionKey", sessionKey);

        return ResponseBean.success(res);
    }


    @PutMapping("/auth-user")
    @ApiOperation(value="授权头像", notes = "授权头像", httpMethod = "PUT", response = TaPerson.class)
    public ResponseBean updateUserInfo(@ApiParam("客户端ID") @PathVariable String clientId,
                                       @ApiParam("授权头像的参数") @RequestBody WxMaAuthParam params) throws Exception {
        WxMaService service = getService(clientId);
        checkAuthParams(service, params);

        // 解密用户信息
        WxMaUserInfo userInfo = service.getUserService().getUserInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        TaPerson taPerson = currentPerson();
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setAvatar(userInfo.getAvatarUrl());
        taPerson.setNickName(userInfo.getNickName());
        taPerson.setSex(Integer.parseInt(userInfo.getGender()));

        iTaPersonService.updateById(taPerson);

        return ResponseBean.success(taPerson);
    }

    @ApiOperation(value="授权手机", notes = "授权手机", httpMethod = "PUT", response = TaPerson.class)
    @PutMapping("/auth-phone")
    public ResponseBean updateUserPhone(@ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("授权手机的参数") @RequestBody WxMaAuthParam params) throws Exception {

        WxMaService service = getService(clientId);

        // 解密
        WxMaPhoneNumberInfo phoneNoInfo = service.getUserService().getPhoneNoInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        TaPerson taPerson = currentPerson();
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setPhone(phoneNoInfo.getPhoneNumber());
        iTaPersonService.updateById(taPerson);

        return ResponseBean.success(taPerson);
    }


    private void checkAuthParams(WxMaService service, WxMaAuthParam params) throws Exception {
        String signature = params.getSignature();
        String rawData = params.getRawData();
        String encryptedData = params.getEncryptedData();
        String iv = params.getIv();
        String sessionKey = params.getSessionKey();

        if (StringUtils.isEmpty(signature) || StringUtils.isEmpty(rawData) || StringUtils.isEmpty(encryptedData) || StringUtils.isEmpty(iv) || StringUtils.isEmpty(sessionKey)) {
            throw new Exception("缺失授权参数");
        }

        boolean isValid = service.getUserService().checkUserInfo(sessionKey, rawData, signature);
        if (!isValid) {
            throw new Exception("校验授权参数失败, 请重试");
        }
    }
}
