package com.yunzhi.nanyang.log;

import ch.qos.logback.core.UnsynchronizedAppenderBase;
import ch.qos.logback.core.db.ConnectionSource;
import ch.qos.logback.core.encoder.Encoder;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.Arrays;

import static ch.qos.logback.core.db.DBHelper.closeStatement;

public class MysqlAppender<E> extends UnsynchronizedAppenderBase<E> {
    protected ConnectionSource connectionSource;
    protected Encoder<E> encoder;
    protected String app = "shigongli";

    @Override
    public void start() {
        if (connectionSource == null) {
            throw new IllegalStateException("MysqlAppender cannot function without a connection source");
        }

        super.start();
    }

    public ConnectionSource getConnectionSource() {
        return connectionSource;
    }

    public void setConnectionSource(ConnectionSource connectionSource) {
        this.connectionSource = connectionSource;
    }

    @Override
    protected void append(E event) {
        Connection connection = null;
        PreparedStatement insertStatement = null;

        try {
            connection = connectionSource.getConnection();
            connection.setAutoCommit(false);
            insertStatement = connection.prepareStatement(getInsertSQL());

            synchronized (this) {
                byte[] logContent = this.encoder.encode(event);
                insertStatement.setBytes(1, logContent);
                insertStatement.execute();
                closeStatement(insertStatement);
            }

            connection.commit();
        } catch (Throwable sqle) {
            addError("problem appending event", sqle);
        }

    }

    public Encoder<E> getEncoder() {
        return encoder;
    }

    public void setEncoder(Encoder<E> encoder) {
        this.encoder = encoder;
    }

    @Override
    public void stop() {
        super.stop();
    }

    private String getInsertSQL() {
        String tableName = "ta_"+app;
        String[] columns = {"log"};
        String[] signs = Arrays.stream(columns).map(x -> "?").toArray(String[]::new);
        String sql = String.format("INSERT INTO %s (%s) values (%s)", tableName, String.join(",", columns), String.join(",", signs));
//        System.out.println(sql);
        return sql;
    }
}
