package com.yunzhi.nanyang.service.impl;

import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.TaAccount;
import com.yunzhi.nanyang.entity.TaAccountLog;
import com.yunzhi.nanyang.entity.TaWithdrawal;
import com.yunzhi.nanyang.mapper.TaAccountLogMapper;
import com.yunzhi.nanyang.mapper.TaAccountMapper;
import com.yunzhi.nanyang.mapper.TaWithdrawalMapper;
import com.yunzhi.nanyang.service.ITaAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * <p>
 * 账户表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaAccountServiceImpl extends BaseServiceImpl<TaAccountMapper, TaAccount> implements ITaAccountService {

    @Autowired
    TaWithdrawalMapper taWithdrawalMapper;

    @Autowired
    TaAccountLogMapper taAccountLogMapper;

    @Override
    public TaAccount getUserAccount(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) return null;

        TaAccount taAccount = getExistBy("user_id", userId, false, false);
        if (null == taAccount) {
            taAccount = new TaAccount();
            taAccount.setUserId(userId);
            save(taAccount);
        }

        return taAccount;
    }

    @Override
    public boolean checkAmount(String orgId, Integer money, Boolean isIncluded) throws Exception {
        TaAccount taAccount = getUserAccount(orgId);
        if (null == money || money <= 0) {
            throw new Exception("校验提款金额失败");
        }

        // 计算所有未审批金额
        Integer totalMoney = taWithdrawalMapper.getTotalMoney(orgId, Constants.AUDIT_READY);
        if (null == totalMoney) {
            totalMoney = 0;
        }

        int amount = money;
        if (null != isIncluded && isIncluded) {
            amount += totalMoney;
        }

        return taAccount.getAmounts() >= amount;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean withdraw(TaWithdrawal taWithdrawal) throws Exception {
        TaAccount taAccount = getUserAccount(taWithdrawal.getOrgId());

        if (taAccount.getAmounts() < taWithdrawal.getMoney()) {
            throw new Exception("账户余额不足");
        }

        TaAccountLog taAccountLog = new TaAccountLog();
        taAccountLog.setAccountId(taAccount.getAccountId());
        taAccountLog.setFeeType(Constants.ACCOUNT_EXPEND);
        taAccountLog.setChargeType(Constants.ACCOUNT_WITHDRAW);
        taAccountLog.setMoney(taWithdrawal.getMoney());
        taAccountLog.setOrderId(taWithdrawal.getWithdrawalId());
        taAccountLog.setCreateDate(LocalDateTime.now());

        if (taAccountLogMapper.insert(taAccountLog) < 1) {
            return false;
        }

        // 余额减少
        int row = baseMapper.updateColumn(taAccount.getAccountId(), "amounts", taAccountLog.getFeeType() * Math.abs(taAccountLog.getMoney()) );
        if (row < 1) {
            throw new Exception("更新账户余额异常");
        }
        // 支出增加
        row = baseMapper.updateColumn(taAccount.getAccountId(), "paid_fee", Math.abs(taAccountLog.getMoney()) );
        if (row < 1) {
            throw new Exception("更新账户支出异常");
        }

        return true;
    }
}
