package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.entity.TaDispatch;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaWorkJob;
import com.yunzhi.nanyang.entity.TaWorkMessage;
import com.yunzhi.nanyang.mapper.TaDispatchMapper;
import com.yunzhi.nanyang.mapper.TaOrderMapper;
import com.yunzhi.nanyang.mapper.TaWorkJobMapper;
import com.yunzhi.nanyang.mapper.TaWorkMessageMapper;
import com.yunzhi.nanyang.service.ITaDispatchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * <p>
 * 调度表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaDispatchServiceImpl extends BaseServiceImpl<TaDispatchMapper, TaDispatch> implements ITaDispatchService {

    @Autowired
    TaWorkJobMapper taWorkJobMapper;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    TaWorkMessageMapper taWorkMessageMapper;

    @Override
    public IPage<TaOrder> getPageBy(IPage<TaOrder> pg, String orgId, String typeId, String machineryId, LocalDateTime start, LocalDateTime end, String phone, String status, Boolean isWarn) {
        return baseMapper.getPageBy(pg, orgId, typeId, machineryId, start, end, phone, status, isWarn);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TaWorkJob createNew(TaDispatch taDispatch, TaOrder taOrder) throws Exception {

        if (!save(taDispatch)) {
            return null;
        }

        TaWorkJob taWorkJob = new TaWorkJob();
        LocalDateTime now = LocalDateTime.now();
        taWorkJob.setJobId(null);
        taWorkJob.setDispatchId(taDispatch.getDispatchId());
        taWorkJob.setOrderId(taOrder.getOrderId());
        taWorkJob.setWorkerId(taDispatch.getWorkerId());
        taWorkJob.setWorkerName(taDispatch.getWorkerName());    // 非数据字段, 但是消息需要用到
        taWorkJob.setMachineryId(taDispatch.getMachineryId());
        taWorkJob.setMachineryName(taDispatch.getMachineryName());    // 非数据字段, 但是消息需要用到
        taWorkJob.setStartDate(now);
        taWorkJob.setOrgId(taDispatch.getOrgId());
        taWorkJob.setStatus(Constants.WORK_READY);
        taWorkJob.setCreateUser(taDispatch.getWorkerId());
        taWorkJob.setCreateDate(now);

        if (taWorkJobMapper.insert(taWorkJob) < 1) {
            throw new Exception("生成作业信息异常");
        }

        if (taOrderMapper.updateDispatchStatus(taOrder.getOrderId(), Constants.DISPATCH_DONE) < 1) {
            throw new Exception("更新订单状态失败");
        }

        try {
            // 生成消息
            TaWorkMessage message = new TaWorkMessage();
            message.setMessageType("accept");
            message.setAcceptPerson(taWorkJob.getWorkerName());
            message.setAcceptTime(taWorkJob.getCreateDate());
            taWorkMessageMapper.insert(message);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return taWorkJob;
    }
}
