package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.TaExtendContent;
import com.yunzhi.nanyang.mapper.TaExtendContentMapper;
import com.yunzhi.nanyang.service.ITaExtendContentService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 拓展内容表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaExtendContentServiceImpl extends BaseServiceImpl<TaExtendContentMapper, TaExtendContent> implements ITaExtendContentService {

    @Override
    public List<TaExtendContent> listByTarget(String targetType, String targetId) {
        QueryWrapper<TaExtendContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("target_type", targetType);
        queryWrapper.eq("target_id", targetId);
        queryWrapper.orderByAsc("sort");

        return list(queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean editContent(String targetType, String targetId, List<TaExtendContent> contentList) throws Exception {
        // 先删除原有数据
        removeByMap(new HashMap<String, Object>(){{
            put("target_type", targetType);
            put("target_id", targetId);
        }});

        if (contentList == null || contentList.isEmpty()) return true;

        for (TaExtendContent content : contentList) {
            content.setTargetType(targetType);
            content.setTargetId(targetId);

            if (StringUtils.isEmpty(content.getExtId())) {
                save(content);
            } else {
                updateById(content);
            }
        }

        return true;
    }
}
