package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.mapper.*;
import com.yunzhi.nanyang.service.ITaMachineryService;
import com.yunzhi.nanyang.vo.ChartParam;
import com.yunzhi.nanyang.vo.MachineSummary;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 农机表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaMachineryServiceImpl extends BaseServiceImpl<TaMachineryMapper, TaMachinery> implements ITaMachineryService {

    @Autowired
    TaImagesMapper taImagesMapper;

    @Autowired
    TdMachineryTypeMapper tdMachineryTypeMapper;

    @Autowired
    TaRegionMapper taRegionMapper;

    @Autowired
    TaExtendContentMapper taExtendContentMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveAll(TaMachinery taMachinery) throws Exception {
        setRedundanceName(taMachinery);

        boolean isOk = save(taMachinery);
        if (!isOk) return false;

        // 图片保存失败不影响主记录
        try {
            saveImages(taMachinery);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    @Override
    public boolean updateAll(TaMachinery taMachinery) throws Exception {
        setRedundanceName(taMachinery);

        boolean isOk = updateById(taMachinery);
        if (!isOk) return false;

        // 图片保存失败不影响主记录
        try {
            saveImages(taMachinery);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    @Override
    public TaMachinery getDetail(String machineryId) {
        TaMachinery taMachinery = getExistBy("machinery_id", machineryId, false, true);
        if (null == taMachinery) return null;

        setRedundanceName(taMachinery);

        List<TaImages> images = taImagesMapper.getListBy(Constants.TARGET_MACHINERY, machineryId);
        taMachinery.setImages(images);

        return taMachinery;
    }

    @Override
    public boolean updateTypeName(String typeId, String typeName) {
        return baseMapper.updateTypeName(typeId, typeName) > 0;
    }

    @Override
    public boolean updateRegionName(String regionId, String regionName) {
        return baseMapper.updateRegionName(regionId, regionName) > 0;
    }

    @Override
    public IPage<MachineSummary> getSummaryList(IPage<MachineSummary> pg, String location, String typeId, String orgId, String q, String clientId) {

        if (Constants.CLIENT_FARMER.equals(clientId)) {
            // 农户端, 实际上过滤的是所有组织下的所有分类的其中一个机器
            return baseMapper.getOneOfTypes(pg, location, typeId, q);
        } else {
            // 农机手端
            return baseMapper.getSummaryList(pg, location, typeId, orgId);
        }

    }

    @Override
    public MachineSummary getSummaryDetail(String machineryId, String location, Boolean attached) {
        MachineSummary summaryDetail = baseMapper.getSummaryDetail(machineryId, location);

        if (attached) {
            List<TaExtendContent> extendContentList = taExtendContentMapper.getByTarget(Constants.TARGET_MACHINERY, machineryId);
            summaryDetail.setContentList(extendContentList);

            List<TaImages> images = taImagesMapper.getListBy(Constants.TARGET_MACHINERY, machineryId);
            summaryDetail.setImagesList(images);
        }

        return summaryDetail;
    }

    @Override
    public boolean updateStatus(String machineryId, String statusColumn, int statusValue) {
        UpdateWrapper<TaMachinery> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set(statusColumn, statusValue);
        updateWrapper.eq("machinery_id", machineryId);

        return update(updateWrapper);
    }

    @Override
    public IPage<TaMachinery> getPageBy(IPage<TaMachinery> pg, String name, String typeId, String orgId, Integer status) {
        return baseMapper.getPageBy(pg, name, typeId, orgId, status);
    }

    @Override
    public ChartParam statisTotal() {
        return baseMapper.statisTotal();
    }

    @Override
    public List<TaMachinery> listByTypeWithLoc(String typeId) {
        return baseMapper.listByTypeWithLoc(typeId);
    }

    private void saveImages(TaMachinery taMachinery) {
        String machineryId = taMachinery.getMachineryId();
        // 先删除所有
        taImagesMapper.deleteByMap(new HashMap<String, Object>(){{
            put("target_id", machineryId);
            put("target_type", Constants.TARGET_MACHINERY);
        }});

        List<TaImages> images = taMachinery.getImages();
        if (null != images && images.size() > 0) {
            for (TaImages image : images) {
                if (!StringUtils.isEmpty(image.getUrl())) {
                    image.setTargetId(machineryId);
                    image.setTargetType(Constants.TARGET_MACHINERY);
                    taImagesMapper.insert(image);
                }
            }
        }
    }

    private void setRedundanceName(TaMachinery taMachinery) {
        // 分类
        if (!StringUtils.isEmpty(taMachinery.getTypeId())) {
            TdMachineryType tdMachineryType = tdMachineryTypeMapper.selectById(taMachinery.getTypeId());
            if (null != tdMachineryType && tdMachineryType.getStatus() > Constants.STATUS_DELETE) {
                taMachinery.setTypeName(tdMachineryType.getName());
            } else {
                taMachinery.setTypeId(null);
                taMachinery.setTypeName(null);
            }
        }

        // 所属区域
        if (!StringUtils.isEmpty(taMachinery.getRegionId())) {
            TaRegion taRegion = taRegionMapper.selectById(taMachinery.getRegionId());
            if (null != taRegion && taRegion.getStatus() > Constants.STATUS_DELETE) {
                taMachinery.setRegionName(taRegion.getName());
            } else {
                taMachinery.setRegionId(null);
                taMachinery.setRegionName(null);
            }
        }
    }
}
