package com.yunzhi.nanyang.service.impl;

import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.event.WorkDispatchEvent;
import com.yunzhi.nanyang.event.WorkExpiredEvent;
import com.yunzhi.nanyang.mapper.TaMessageMapper;
import com.yunzhi.nanyang.mapper.TaPersonMapper;
import com.yunzhi.nanyang.service.ITaMessageService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.nanyang.service.ITaMessageTemplateService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 消息表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Slf4j
@Service
public class TaMessageServiceImpl extends BaseServiceImpl<TaMessageMapper, TaMessage> implements ITaMessageService {

    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    ITaMessageTemplateService iTaMessageTemplateService;

    @Autowired
    WxUtils wxUtils;

    @Autowired
    SMSUtils smsUtils;

    @Async
    @Override
    public void sendMessage(WorkExpiredEvent evt) {
        evt.getTaDispatch();
    }

    @Async
    @Override
    public void sendMessage(WorkDispatchEvent evt) {

        try {
            TaWorkJob taWorkJob = evt.getTaWorkJob();
            String jobId = taWorkJob.getJobId();
            TaOrder taOrder = evt.getTaOrder();
            String orderNo = taOrder.getOrderNo();
            TaPerson taPerson = taPersonMapper.getByUserId(taWorkJob.getWorkerId());
            if (null == taPerson) {
                log.error("发送消息失败, 未找到对应的人员信息 " + taWorkJob.getWorkerId());
                return;
            }

            // 先发送小程序
            try {
                TaMessageTemplate tpl = iTaMessageTemplateService.getTemplateByType(Constants.MESSAGE_MINIAPP, Constants.MESSAGE_ORDER_NOTIFY);
                WxMaSubscribeMessage message = MessageUtil.getWxMaSubscribeMessage(tpl);
                if (null != message) {
                    message.setToUser(taPerson.getOpenid());
                    String actionStr = tpl.getAppTplAction();
                    message.setPage(MessageUtil.NaviPage.fromJSON(actionStr).toURL(jobId));
                    List<WxMaSubscribeMessage.MsgData> data = MessageUtil.getWxMessageData(tpl.getAppTplContent(),
                            orderNo,
                            taWorkJob.getMachineryName(),
                            "已派单",
                            DateUtils.toString(taWorkJob.getCreateDate(), "yyyy-MM-dd HH:mm"));
                    message.setData(data);

                    wxUtils.getWorkerService().getMsgService().sendSubscribeMsg(message);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            // 再发送短信
            try {
                TaMessageTemplate tpl = iTaMessageTemplateService.getTemplateByType(Constants.MESSAGE_SMS, Constants.MESSAGE_ORDER_NOTIFY);
                SMSUtils.Message message = MessageUtil.getSmsMessage(tpl);
                if (!StringUtils.isEmpty(taPerson.getPhone()) && null != message) {
                    message.setTel(taPerson.getPhone());
                    message.setContent(String.format(tpl.getAppTplContent(), taWorkJob.getWorkerName(), orderNo));

                    smsUtils.sendMessage(message);
                } else {
                    log.error("发送短信失败, 可能无用户手机 " + taPerson.getPersonId());
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
