package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.entity.TaAccount;
import com.yunzhi.nanyang.entity.TaAccountLog;
import com.yunzhi.nanyang.entity.TaWithdrawal;
import com.yunzhi.nanyang.mapper.TaAccountLogMapper;
import com.yunzhi.nanyang.mapper.TaAccountMapper;
import com.yunzhi.nanyang.mapper.TaWithdrawalMapper;
import com.yunzhi.nanyang.service.ITaAccountService;
import com.yunzhi.nanyang.service.ITaWithdrawalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * <p>
 * 提现表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaWithdrawalServiceImpl extends BaseServiceImpl<TaWithdrawalMapper, TaWithdrawal> implements ITaWithdrawalService {

    @Autowired
    ITaAccountService iTaAccountService;

    @Autowired
    TaAccountLogMapper taAccountLogMapper;

    @Autowired
    TaAccountMapper taAccountMapper;

    @Override
    public IPage<TaWithdrawal> getPageBy(IPage<TaWithdrawal> pg, String orgId, LocalDateTime startDate, LocalDateTime endDate, String userName, Integer auditStatus) {
        return baseMapper.getPageBy(pg, orgId, startDate, endDate, userName, auditStatus);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean audit(TaWithdrawal taWithdrawal) throws Exception {

        // 只有审核同意, 才需要处理账户信息
        if (taWithdrawal.getAuditStatus() == Constants.AUDIT_AGREE) {
            TaAccount taAccount = iTaAccountService.getUserAccount(taWithdrawal.getOrgId());

            if (taAccount.getAmounts() < taWithdrawal.getMoney()) {
                throw new Exception("账户余额不足");
            }

            TaAccountLog taAccountLog = new TaAccountLog();
            taAccountLog.setAccountId(taAccount.getAccountId());
            taAccountLog.setFeeType(Constants.ACCOUNT_EXPEND);
            taAccountLog.setChargeType(Constants.ACCOUNT_WITHDRAW);
            taAccountLog.setMoney(taWithdrawal.getMoney());
            taAccountLog.setOrderId(taWithdrawal.getWithdrawalId());
            taAccountLog.setCreateDate(LocalDateTime.now());

            if (taAccountLogMapper.insert(taAccountLog) < 1) {
                return false;
            }

            // 余额减少
            int row = taAccountMapper.updateColumn(taAccount.getAccountId(), "amounts", taAccountLog.getFeeType() * Math.abs(taAccountLog.getMoney()) );
            if (row < 1) {
                throw new Exception("更新账户余额异常");
            }
            // 支出增加
            row = taAccountMapper.updateColumn(taAccount.getAccountId(), "paid_fee", Math.abs(taAccountLog.getMoney()) );
            if (row < 1) {
                throw new Exception("更新账户支出异常");
            }
        }

        return updateById(taWithdrawal);
    }
}
